
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include "cdhcp.h"


void CDhcp :: 
Initialize()
{
	memset(&m_Hdr, 0, sizeof(m_Hdr));
	memset(&m_pucOpts, 0, sizeof(m_pucOpts));
	memset(&m_pucOpts43, 0, sizeof(m_pucOpts43));

	m_pExtendedPxeClientTag = NULL;
	m_pOptTag93 = NULL;
	m_bOptTag93Same = TRUE;

	m_pOptTag94 = NULL;
	m_bOptTag94Same = TRUE;

}// OBJECT CONSTRUCTORS
// Create a new CDhcp object of the minimum size initialized to zero
CDhcp :: 
CDhcp()
{
	m_OptionSize = MINPKTSIZE - HDRSIZE;
	m_OptionsUsed = MINOPTUSED;  // Allow for Magic Cookie

	Initialize();
	m_Hdr.magic = MAGIC_COOKIE;
}

// Create a new CDhcp object of size p_usSize initialized to zero
CDhcp :: 
CDhcp(USHORT p_usSize)
{
	if (p_usSize >= MINPKTSIZE)
		m_OptionSize = p_usSize - HDRSIZE;
	else
		m_OptionSize = MINPKTSIZE - HDRSIZE;

	m_OptionsUsed = MINOPTUSED;

	Initialize();

	m_Hdr.magic = MAGIC_COOKIE;
}

// Create a new CDhcp object of size p_usSize.  p_pucData is a byte
// array containing a DHCP packet.
CDhcp :: 
CDhcp(const BYTE* p_pucData,  USHORT p_usSize)
{
	BYTE ucOptionOverload = 0;
	BOOL bMoveSname = FALSE;
	BOOL bMoveFilename = FALSE;
	// Pointer to the option portion of the packet located after the magic cookie
	const BYTE* pucOpts = (p_pucData + HDRSIZE + COOKIESIZE);
	// Maximum number of bytes in the options not counting the magic cookie
	USHORT usOptLen = p_usSize - (USHORT)(HDRSIZE + COOKIESIZE);
	// Index of start of current option.
	USHORT usIdx = 0;
	// Length of current option (pucOpts[usIdx+1]+2).
	BYTE ucLen = 0;

	Initialize();

	m_OptionsUsed = MINOPTUSED;
	if (p_usSize >= MINPKTSIZE)
		m_OptionSize = p_usSize - HDRSIZE;
	else
		m_OptionSize = MINPKTSIZE - HDRSIZE;
	// Copy the header and the magic cookie
	memcpy(&m_Hdr, p_pucData, HDRSIZE + COOKIESIZE);

	// Copy the options
	// GetOption will return 0 if trailing FF is found or usOptLen is exceeded.
	// The loop is executed first for the normal options.  Then if option 52, option
	// overload, is found it will excute again for the sname and/or filename fields.
	// If option 52 is found the corresponding values will be moved from the option
	// array to the header fields so the other Methods will work correctly.
	do
	{
		while (GetNextOption(pucOpts, usIdx, usOptLen))
		{
			ucLen = pucOpts[usIdx+1] +2;
			// Look for Option Overload where data is stored in the sname and filename fields.
			if (pucOpts[usIdx] == 52) 
			{
				// Save value to process later
				ucOptionOverload = pucOpts[usIdx+2];
				// Still add the option to the option array.
			}	

			StoreOption(pucOpts, usIdx, usOptLen);
			// Index to next option
			usIdx += ucLen;
		}
		if (ucOptionOverload == 1 || ucOptionOverload == 3)
		{	// Process options in sname field.
			pucOpts = m_Hdr.sname;
			usIdx = 0;
			usOptLen = 64;	// Max length of sname field.
			// Move sname from option 66 to sname field
			ucOptionOverload -= 1;
			bMoveSname = TRUE;

		} else if (ucOptionOverload == 2)
		{	// Process options in filename field.
			pucOpts = m_Hdr.bootfile;
			usIdx = 0;
			usOptLen = 128;	// Max length of filename.
			// Move filename from option 67 to filename field.
			ucOptionOverload -= 2;
			bMoveFilename = TRUE;
		} else
		{	// No more options
			if (bMoveSname)
			{
				memset(m_Hdr.sname, 0, 64);
				if(m_pucOpts[66])
					memcpy(m_Hdr.sname, m_pucOpts[66], (m_pucOpts[66])[1]);
			}
			if (bMoveFilename)
			{
				memset(m_Hdr.bootfile, 0, 128);
				if(m_pucOpts[67])
					memcpy(m_Hdr.bootfile, m_pucOpts[67], (m_pucOpts[67])[1]);
			}
				
			break;
		}
	} while (1);
}

// Create a new CDhcp  object the same size as p_pucDhcpPacket and copy
// p_pucDhcpPacket to the new object.
CDhcp :: 
CDhcp(const CDhcp& p_pucDhcpPacket)
{
	m_OptionsUsed = p_pucDhcpPacket.m_OptionsUsed;
	m_OptionSize = p_pucDhcpPacket.m_OptionSize;

	Initialize();

	memcpy(&m_Hdr, &p_pucDhcpPacket.m_Hdr, sizeof(m_Hdr));
	for (int i=0; i<255; i++)
	{
		if (p_pucDhcpPacket.m_pucOpts[i])
		{
			BYTE ucLen = *(p_pucDhcpPacket.m_pucOpts[i]+1);
			m_pucOpts[i] = new BYTE[ucLen+2];
			memcpy(m_pucOpts[i], p_pucDhcpPacket.m_pucOpts[i], ucLen+2);
		}
		if (p_pucDhcpPacket.m_pucOpts43[i])
		{
			BYTE ucLen = *(p_pucDhcpPacket.m_pucOpts43[i]+1);
			m_pucOpts43[i] = new BYTE[ucLen+2];
			memcpy(m_pucOpts43[i], p_pucDhcpPacket.m_pucOpts43[i], ucLen+2);
		}

	}
}

//Object Destructor
CDhcp ::
~CDhcp()
{
	// Release allocated memory.
	for (int i=0; i<255; i++)
	{
		if (m_pucOpts[i])
			delete[] m_pucOpts[i];
		if (m_pucOpts43[i]) 
			delete[] m_pucOpts43[i];
	}
	if (m_pExtendedPxeClientTag)
		delete m_pExtendedPxeClientTag;

	if (m_pOptTag94)
		delete [] m_pOptTag94;

	if (m_pOptTag93)
		delete [] m_pOptTag93;
};

// OPERATORS
// Assignment
const CDhcp& CDhcp::
operator =(const CDhcp& p_Pkt)
{
	int i;
	// Copy the simple member variables
	m_OptionsUsed = p_Pkt.m_OptionsUsed;
	m_OptionSize = p_Pkt.m_OptionSize;

	// Copy the header portion of the packet
	memcpy(&m_Hdr, &p_Pkt.m_Hdr, sizeof(m_Hdr));

	// Delete all of the old memory allocated for options.
	for (i=0; i<255; i++)
	{
		if (m_pucOpts[i])
			delete[] m_pucOpts[i];
		if (m_pucOpts43[i])
			delete[] m_pucOpts[i];
	}

	// Copy all of the new options
	for ( i=0; i<255; i++)
	{
		if (p_Pkt.m_pucOpts[i])
		{
			BYTE ucLen = *(p_Pkt.m_pucOpts[i]+1);
			m_pucOpts[i] = new BYTE[ucLen+2];
			memcpy(m_pucOpts[i], p_Pkt.m_pucOpts[i], ucLen+2);
		}
		if (p_Pkt.m_pucOpts43[i])
		{
			BYTE ucLen = *(p_Pkt.m_pucOpts43[i]+1);
			m_pucOpts43[i] = new BYTE[ucLen+2];
			memcpy(m_pucOpts43[i], p_Pkt.m_pucOpts43[i], ucLen+2);
		}

	}

	return *this;
}

// STATE QUERIES
// Each of these returns the value in the specified field.  All multi-byte values are
// returned in network order.
BYTE CDhcp ::
Opcode()
{
	return (m_Hdr.opcode);
}

BYTE CDhcp ::
Hardware()
{
	return (m_Hdr.hardware);
}

BYTE CDhcp ::
HardwareLength()
{
	return (m_Hdr.hardlen);
}

BYTE CDhcp ::
GatewayHops()
{
	return (m_Hdr.gatehops);
}

ULONG CDhcp ::
TransactionID()
{
	return (m_Hdr.ident);
}

USHORT CDhcp ::
Seconds()
{
	return (m_Hdr.seconds);
}

USHORT CDhcp ::
Flags()
{
	return (m_Hdr.flags);
}

ULONG CDhcp ::
ClientIP()
{
	return (m_Hdr.cip);
}

ULONG CDhcp ::
YourIP()
{
	return (m_Hdr.yip);
}

ULONG CDhcp ::
ServerIP()
{
	return (m_Hdr.sip);
}

ULONG CDhcp ::
GatewayIP()
{
	return (m_Hdr.gip);
}

BYTE CDhcp::
HardwareAddress(BYTE* p_ucData, int p_nSize)
{
	if (p_nSize < m_Hdr.hardlen)
		return 0;
	if (m_Hdr.hardlen == 0)
		return 0;

	memcpy(p_ucData, m_Hdr.caddr, m_Hdr.hardlen);
	return m_Hdr.hardlen;
}

BYTE CDhcp ::
ServerName(BYTE *p_ucName, int p_nSize)
{
	if (p_nSize >= 64)
	{
		memcpy(p_ucName, m_Hdr.sname, 64);
		return (64);
	} else
		return 0;
}

BYTE CDhcp ::
BootFileName(BYTE *p_ucName, int p_nSize)
{
	if (p_nSize >= 128)
	{
		memcpy(p_ucName, m_Hdr.bootfile, 128);
		return (128);
	} else
		return 0;
}

ULONG CDhcp ::
MagicCookie()
{
	return (m_Hdr.magic);
}

// Return the current length of the packet in use.
USHORT CDhcp :: PacketLength()
{
	return (m_OptionsUsed + HDRSIZE);
}

//Return the maximum number of bytes the packet is allowed to hold.
USHORT CDhcp :: PacketSize()
{
	return (m_OptionSize + HDRSIZE);
}

//Return number of bytes allocated for the option portion of the packet including
//the magic cookie.
USHORT CDhcp :: OptionsSize()
{
	return(m_OptionSize);
}

//Return the number of bytes currently used in the option portion of the packet,
// including the magic cookie.
USHORT CDhcp :: OptionsLength()
{
	return(m_OptionsUsed);
}

//Return the number of bytes remaining in the options portion of the packet.
USHORT CDhcp :: OptionsAvailableLength()
{
	return(m_OptionSize - m_OptionsUsed);
}
// Copy contents of the CDhcp object to p_pucData as a BYTE array.  
// On call, p_pnSize contains the number of BYTES available in p_pbData.  On return 
// p_pnSize contains number of bytes copied to p_pucData.
// Return TRUE if successful. Return FALSE if p_pnSize is too small to contain
// the packet.
BOOL CDhcp :: GetPacket(BYTE* p_pucData, int& p_pnSize)
{
	// Index into the output byte array
	USHORT usIdx = 0;
	BYTE ucOptLen = 0;
	USHORT us43LenPos;
	BYTE uc43Len;
	int i;

	if (p_pnSize <= (HDRSIZE + m_OptionsUsed))
		return FALSE;

	// Allocate temp buffer with excess space for possible overhead.
	// This is to ensure we have a large enough buffer so we don't have to keep checking
	// to see if we are exceeding the buffer size.
	BYTE* pucTmp = new BYTE[HDRSIZE + m_OptionsUsed + 500];
	// Ensure all bytes are zero in case less then 300 bytes are used.
	memset(pucTmp, '\0', HDRSIZE + m_OptionsUsed + 500);

	// Copy the header and magic cookie
	memcpy(&pucTmp[usIdx], &m_Hdr, HDRSIZE + COOKIESIZE);
	usIdx = HDRSIZE + COOKIESIZE;
	// Format and copy the all of the options
	if (m_OptionsUsed > MINOPTUSED)
	{
		// Copy the non-43 options first
		for(i=0; i<255; i++)
		{
			if (m_pucOpts[i])
			{
				// Get length of option from second byte, add 2 for tag and len bytes.
				ucOptLen = *(m_pucOpts[i] + 1) + 2;
				memcpy(&pucTmp[usIdx], m_pucOpts[i], ucOptLen);
				usIdx += ucOptLen;
			}
		}

		// Copy the 43 type encapsulated options
		// Start new 43 Option
		pucTmp[usIdx++] = 43;
		us43LenPos = usIdx++;
		ucOptLen = 0;
		for(i=0; i<255; i++)
		{
			if (m_pucOpts43[i])
			{
				// Get length of option from second byte, add 2 for tag and len bytes.
				uc43Len = *(m_pucOpts43[i] + 1) + 2;
				// If this option full, start another.
				if (ucOptLen + uc43Len > 254)
				{
					// Close the 43 option.
					pucTmp[usIdx++] = 0xFF;
					pucTmp[us43LenPos] = ucOptLen + 1;
					// Start a new 43 option.
					pucTmp[usIdx++] = 43;
					us43LenPos = usIdx++;
					ucOptLen = 0;
				}
				memcpy(&pucTmp[usIdx], m_pucOpts43[i], uc43Len);
				usIdx += uc43Len;
				ucOptLen += uc43Len;
			}
		}

		if (ucOptLen == 0)
		{
			// There were no 43 options so backup the index.
			usIdx -= 2;
		} else
		{
			// There were type 43 encapsulated options so close the option.
			// Close the 43 option.
			pucTmp[usIdx++] = 0xFF;
			pucTmp[us43LenPos] = ucOptLen + 1;
		}

		// Close all of the options
		pucTmp[usIdx++] = 0xFF;
	}

	// Set minimum packet size for forwarding through 
	// Bay Network Routers.
	if (p_pnSize < 300) p_pnSize = 300;

	if (usIdx < 300) usIdx = 300;

	if (usIdx <= p_pnSize)
	{
		memcpy(p_pucData, pucTmp, usIdx);
		p_pnSize = usIdx;
		delete[] pucTmp;
		return TRUE;
	} else
	{
		delete[] pucTmp;
		return FALSE;
	}
}

//Copy only the data for the option specified by p_ucTag from the option portion of the packet
//to p_pucData.  p_nSize indicates the number of bytes available.  Return the number of bytes
//copied.  If p_nSize is too small return 0.  If the option is not found return 0.
//The length of the data for the option can be determined by setting p_pucData to NULL
// and p_nSize to 0.  The number of bytes in the option data is returned.  If the option
//is not found 0 is returned.
//If there are multiple options of the same type, the data for the last one found is
//returned.
int CDhcp :: GetOption(BYTE p_ucTag, BYTE* p_pucData, int p_nSize)
{
	BYTE ucOptLen;

	// Just asking for the length of the option?
	if ((p_pucData == NULL) && (p_nSize ==0))
	{
		if (m_pucOpts[p_ucTag])
		{
			ucOptLen = *(m_pucOpts[p_ucTag] +1);
			return ucOptLen;
		} else
		{
			return 0;
		}
	}	

	if (m_pucOpts[p_ucTag])
	{
		ucOptLen = *(m_pucOpts[p_ucTag] +1);
		// Verify room for the option.
		if (p_nSize < ucOptLen)
			return -ucOptLen;
		memcpy(p_pucData, m_pucOpts[p_ucTag]+2, ucOptLen);
		return ucOptLen;
	} else
	{
		return 0;
	}

}

//Copy only the data for the 43 sub-option specified by p_ucTag from the option portion of
//the packet to p_pucData.  p_nSize indicates the number of bytes available.  Return the
//number of bytes copied.  If p_nSize is too small return 0.  If the option is not found
//return 0.
//The length of the data for the option can be determined by setting p_pucData to NULL
//and p_nSize to 0.  The number of bytes in the option data is returned.  If the option
//is not found 0 is returned.
//All 43 options will be searched for the sub-option.  If there are multiple sub-options
//of the same type, the data for the first one found is returned.
int CDhcp :: Get43Option(BYTE p_ucTag, BYTE* p_pucData, int p_nSize)
{
	BYTE ucOptLen;

	// Just asking for the length of the option?
	if ((p_pucData == NULL) && (p_nSize ==0))
	{
		if (m_pucOpts43[p_ucTag])
		{
			ucOptLen = *(m_pucOpts43[p_ucTag] +1);
			return ucOptLen;
		} else
		{
			return 0;
		}
	}	

	if (m_pucOpts43[p_ucTag])
	{
		ucOptLen = *(m_pucOpts43[p_ucTag] +1);
		// Verify room for the option.
		if (p_nSize < ucOptLen)
			return -ucOptLen;
		memcpy(p_pucData, m_pucOpts43[p_ucTag]+2, ucOptLen);
		return ucOptLen;
	} else
	{
		return 0;
	}

}

//STATE CONSTRUCTORS
//Each of these functions sets the specified field to the value provided.  The caller
//must perform any conversions to network order.
void CDhcp :: Opcode(BYTE p_ucCode)
{
	m_Hdr.opcode = p_ucCode;
}

void CDhcp :: Hardware(BYTE p_ucHw)
{
	m_Hdr.hardware = p_ucHw;
}

void CDhcp :: HardwareLength(BYTE p_ucHwLen)
{
	m_Hdr.hardlen = p_ucHwLen;
}

void CDhcp :: GatewayHops(BYTE p_ucHops)
{
	m_Hdr.gatehops = p_ucHops;
}

void CDhcp :: TransactionID(ULONG p_ulId)
{
	m_Hdr.ident = p_ulId;
}

void CDhcp :: Seconds(USHORT p_usSecs)
{
	m_Hdr.seconds = p_usSecs;
}

void CDhcp :: Flags(USHORT p_usFlags)
{
	m_Hdr.flags = p_usFlags;
}

void CDhcp :: ClientIP(ULONG p_ulCiaddr)
{
	m_Hdr.cip = p_ulCiaddr;
}

void CDhcp :: YourIP(ULONG p_ulYiaddr)
{
	m_Hdr.yip = p_ulYiaddr;
}

void CDhcp :: ServerIP(ULONG p_ulSiaddr)
{
	m_Hdr.sip = p_ulSiaddr;
}

void CDhcp :: GatewayIP(ULONG p_ulGiaddr)
{
	m_Hdr.gip = p_ulGiaddr;
}

void CDhcp::
SetHardwareAddress(BYTE* p_ucData, int p_nSize)
{
	memset(m_Hdr.caddr, 0, 16);
	memcpy(m_Hdr.caddr, p_ucData,  MIN(p_nSize, 16) );
	m_Hdr.hardlen = (BYTE) MIN(p_nSize, 16);
}

void CDhcp :: SetServerName(BYTE *p_ucName, int p_nSize)
{
	memset(m_Hdr.sname, 0, 64);
	memcpy(m_Hdr.sname, p_ucName, (p_nSize >64)?64: p_nSize);
}

void CDhcp :: SetBootFileName(BYTE *p_ucName, int p_nSize)
{
	memset(m_Hdr.bootfile, 0, 128);
	memcpy(m_Hdr.bootfile, p_ucName, (p_nSize >128)?128: p_nSize);
}

void CDhcp :: MagicCookie(ULONG p_ulCookie)
{
	m_Hdr.magic = p_ulCookie;
}

// Insert the option specified by p_ucTag.  Return TRUE if successful.  Return FALSE if there
// is not enough room left in the packet and leave the packet unchanged.  Will overwrite 
// curent option value.
BOOL CDhcp :: AddOption(BYTE p_ucTag, BYTE* p_pucData, int nSize)
{
	BYTE*	pucTmp;
	BYTE	ucSize;


	// If the option is currently defined and the new option will fit in the available space
	// then delete the old option.  If the new option will not fit then leave the current
	// option unchanged and return FALSE.
	if (m_pucOpts[p_ucTag])
	{
		ucSize = (m_pucOpts[p_ucTag])[1];
		if (m_OptionsUsed + nSize - ucSize > m_OptionSize)
			return FALSE;
		else
			// DelOption updates m_OptionsUsed.
			DelOption(p_ucTag);

	} else if (m_OptionsUsed + nSize > m_OptionSize)
	{
		return FALSE;
	}

	// Copy and insert the new option.
	pucTmp = new BYTE[nSize+2];
	pucTmp[0] = p_ucTag;
	pucTmp[1] = (BYTE) nSize;
	memcpy(&pucTmp[2], p_pucData, nSize);
	m_pucOpts[p_ucTag] = pucTmp;
	// nSize is data length plus 2 for tag and length bytes.
	m_OptionsUsed += (nSize + 2);
	return TRUE;
}

//Delete the option specified by p_ucTag if the option has been previously defined.
void CDhcp ::
DelOption(BYTE p_ucTag)
{
	BYTE ucSize;

	if (m_pucOpts[p_ucTag])
	{
		ucSize = (m_pucOpts[p_ucTag])[1];
		// nSize is data length plus 2 for tag and length bytes.
		m_OptionsUsed -= (ucSize + 2);
		delete [] m_pucOpts[p_ucTag];
		m_pucOpts[p_ucTag] = NULL;
	}

}

//Add an encapsulated vendor specific information into a type 43 vendor specified information
//option.  Insert the sub-option specified by p_ucTag into a 43 option.  Return TRUE if
//successful.  Return FALSE if there is not enough room left in the packet.
//Encapsulated vendor specific information will be packed into type 43 options with a
//maximum length of 255.  The order the options are inserted may not be the same order the
//options are placed into the type 43 options.
//Multiple type 43 options will be created if the length exceeds 255.
BOOL CDhcp :: Add43Option(BYTE p_ucTag, BYTE* p_pucData, int nSize)
{
	BYTE*	pucTmp;
	BYTE	ucSize;


	// If the option is currently defined and the new option will fit in the available space
	// then delete the old option.  If the new option will not fit then leave the current
	// option unchanged and return FALSE.
	if (m_pucOpts[p_ucTag])
	{
		ucSize = (m_pucOpts43[p_ucTag])[1];
		if (m_OptionsUsed + nSize - ucSize > m_OptionSize)
			return FALSE;
		else
			// DelOption updates m_OptionsUsed.
			Del43Option(p_ucTag);

	} else if (m_OptionsUsed + nSize > m_OptionSize)
	{
		return FALSE;
	}

	pucTmp = new BYTE[nSize+2];
	pucTmp[0] = p_ucTag;
	pucTmp[1] = (BYTE) nSize;
	memcpy(&pucTmp[2], p_pucData, nSize);
	m_pucOpts43[p_ucTag] = pucTmp;
	m_OptionsUsed += (nSize +2);
	return TRUE;

}

//Delete the option specified by p_ucTag if the option has been previously defined.
void CDhcp ::
Del43Option(BYTE p_ucTag)
{
	BYTE ucSize;

	if (m_pucOpts43[p_ucTag])
	{
		ucSize = (m_pucOpts43[p_ucTag])[1];
		// nSize is data length plus 2 for tag and length bytes.
		m_OptionsUsed -= (ucSize + 2);
		delete [] m_pucOpts43[p_ucTag];
		m_pucOpts43[p_ucTag] = NULL;
	}

}

//Set the available length of the options portion of the packet.  If  p_usSize is larger
//then more space is allocated as needed.  Previous option data is preserved.  If p_usSize
//is smaller and the current the number of bytes in the current data exceeds the new
//p_usSize, the packet remains unchanged and FALSE is returned.  FALSE is also returned
//if p_nSize is smaller then 312 the minimum size allowed for the options in a DHCP
//packet (RFC 2131).
//Returns TRUE if successful otherwise returns FALSE.
BOOL CDhcp :: SetSize(USHORT p_usSize)
{
	if (p_usSize < m_OptionsUsed + HDRSIZE)
		return FALSE;

	if (p_usSize < MINPKTSIZE)
		return FALSE;
	m_OptionSize = p_usSize - HDRSIZE;
	return TRUE;
}


// Return the index of the first option found in the buffer pointed at by p_pucOpts
// If the end of the buffer is reached return 0.  End of options is detected by an 0xFF
// or by exceeding the usOptLen value.
BOOL CDhcp::
GetNextOption(const BYTE* p_pucOpts, USHORT& p_usIdx, USHORT p_usOptlen)
{
	while(1)
	{
		if (p_usIdx >= p_usOptlen)
			return FALSE;

		if (p_pucOpts[p_usIdx] == 0)
			p_usIdx++;
		else if (p_pucOpts[p_usIdx] == 0xFF)
			return FALSE;
		else 
			return TRUE;
	}

}
//////////////////////////
void CDhcp::
StoreOption(const BYTE* pucOpts, const USHORT usIdx, const USHORT usOptLen)
{
	BYTE ucLen;

	// Length of option data plus 2 for Option Code and Len fields.
	ucLen = pucOpts[usIdx+1] + 2;

	if (pucOpts[usIdx] != 43)
	{
		// Make sure we have a valid option to copy
		if ((ucLen + usIdx) < usOptLen)
		{
			if (pucOpts[usIdx] == 60 && ucLen > 12)
			{
				HandleNewClassIdentifier(pucOpts, usIdx, ucLen);
				// this is the extended class identifier
			}
			else
			if (!m_pucOpts[pucOpts[usIdx]])
			{
				m_pucOpts[pucOpts[usIdx]] = new BYTE[ucLen];
				memcpy(m_pucOpts[pucOpts[usIdx]], &pucOpts[usIdx], ucLen);
				m_OptionsUsed += ucLen;
			}
			else
			{
				checkTags(m_pucOpts[pucOpts[usIdx]], (BYTE*)&pucOpts[usIdx], ucLen);
			}
		}

	} else // Parse the type 43 sub-options
	{
		USHORT usIdx2 = 0;
		const BYTE* pucOpts2 = &pucOpts[usIdx + 2];
		BYTE ucLen2 = 0;
		while ((GetNextOption(pucOpts2, usIdx2, ucLen)))
		{
			ucLen2 = pucOpts2[usIdx2+1] + 2;
			m_pucOpts43[pucOpts2[usIdx2]] = new BYTE[ucLen];
			memcpy(m_pucOpts43[pucOpts2[usIdx2]], &pucOpts2[usIdx2], ucLen2);
			usIdx2 += ucLen2;
			m_OptionsUsed += ucLen2;
		}

	}
}

BOOL CDhcp::
HandleNewClassIdentifier(const BYTE *pucOpts, const USHORT usIdx, USHORT ucLen)
{
#define PXE_CLS_CLIENT "PXEClient"

	char text[512];
	char buf[512];
	char seps[] = ":"; // separator in between the class identifier
	BYTE *ptr;

	// Make a copy of the options string in 'text'
	memcpy(text, &pucOpts[usIdx+2], ucLen-2);
	text[ucLen-2] = 0; // Null terminate the string
	m_pExtendedPxeClientTag = new BYTE[ucLen];
	memcpy((char*)m_pExtendedPxeClientTag, text, ucLen-1); // includes NULL
	
	char *token = strtok( text, seps );   
	if (!token)
		return FALSE;

	USHORT usClassLen = strlen(PXE_CLS_CLIENT);
	// If the start of the tag is not "PXEClient", then that is an error
	if (strncmp(text, PXE_CLS_CLIENT, usClassLen))
		return FALSE;

	// Now, create an option 60 which is just "PXEClient" and not the extended string
	if (m_pucOpts[60])
		delete [] m_pucOpts[60];
	ptr = m_pucOpts[60] = new BYTE[usClassLen+2];
	ptr[0] = 60;
	ptr[1] = (BYTE)usClassLen;
	memcpy(&ptr[2], PXE_CLS_CLIENT, usClassLen);
	m_OptionsUsed += 11;


	// The next part is the client architecture
	token = strtok( NULL, seps );   
	if (!token)
		return FALSE;
	if (strncmp(token, "Arch", 4 ))
		return FALSE;

	token = strtok( NULL, seps );   
	if (!token)
		return FALSE;
	int nArch;
	sscanf(token, "%d", &nArch);

	// Now, create an option 93 tag
	ptr = new BYTE[4];
	ptr[0] = 93;
	ptr[1] = 2;
	*(USHORT*)&ptr[2] = htons(nArch);
	m_OptionsUsed += 4;

	if (m_pucOpts[93])
	{
		checkTags(ptr, m_pucOpts[93], 4);
		delete [] m_pucOpts[93];
	}

	m_pucOpts[93] = ptr;

	// Now, get the NIC version

	token = strtok(NULL, seps );   
	if (!token)
		return FALSE;
	if (strncmp(token, "UNDI", 4 ))
		return FALSE;

	token = strtok(NULL, seps );
	int nMinor, nMajor;
	sscanf(token, "%3d%3d", &nMajor, &nMinor);

	ptr = new BYTE[5];
	ptr[0] = 94;
	ptr[1] = 3;
	ptr[2] = 1; //NIC type is UNDI which is value '1'
	ptr[3] = nMajor;
	ptr[4] = nMinor;
	m_OptionsUsed += 5;

	if (m_pucOpts[94])
	{
		checkTags(ptr, m_pucOpts[94], 5);
		delete [] m_pucOpts[94];
	}

	m_pucOpts[94] = ptr;

	return TRUE;

}

void CDhcp::
checkTags(BYTE* pExtendedTag, BYTE* pOldTypeTag, int len)
{
	if (pExtendedTag == NULL || pOldTypeTag == NULL)
		return;

	char buf[1024];

	BOOL bSame = (memcmp(pExtendedTag, pOldTypeTag, len) == 0);

	BYTE *ptr = new BYTE[len];
	memcpy(ptr, pOldTypeTag, len);
	if (ptr[0] == 93)
	{
		m_pOptTag93 = ptr;
		m_bOptTag93Same = bSame;
	}
	else
	if (ptr[0] == 94)
	{
		m_pOptTag94 = ptr;
		m_bOptTag94Same = bSame;
	}
}

BOOL CDhcp::
GetOldStyleOpt93(CString *pcs, BOOL* pbSame)
{
	if (m_pOptTag93)
	{
		*pbSame = m_bOptTag93Same;
		*pcs = (char*)m_pOptTag93;
		return TRUE;
	}
	else
	{
		return FALSE;
	}
	return FALSE; // never would reach this path

}

BOOL CDhcp::
GetOldStyleOpt94(CString *pcs, BOOL* pbSame)
{
	if (m_pOptTag94)
	{
		*pbSame = m_bOptTag94Same;
		*pcs = (char*)m_pOptTag94;
		return TRUE;
	}
	else
	{
	}
	return FALSE; 

}

BOOL CDhcp::
GetExtendedPxeClientTag(CString *pcs)
{
	if (m_pExtendedPxeClientTag)
	{
		*pcs = (char*)m_pExtendedPxeClientTag;
		return TRUE;
	}
	return FALSE;

}
