
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include "cipaddr.h"
#include <ctype.h>

CIPAddr::CIPAddr()
{
	_ipoctet[0] = 0;
	_ipoctet[1] = 0;
	_ipoctet[2] = 0;
	_ipoctet[3] = 0;
}

CIPAddr::CIPAddr(const CString &ipaddr)
{
	SetIP(ipaddr);
}

CIPAddr::CIPAddr(DWORD ipaddr, bool networkbyteorder)
{
	SetIP(ipaddr, networkbyteorder);
}

CIPAddr::CIPAddr(const CIPAddr &cipaddr)
{
	SetIP(cipaddr);
}

// make sure that the CString that is being passed is trimmed
// Left and Rright to remove all the white spaces around the 
// IP Address string
void 
CIPAddr::SetIP(const CString &ipaddr)
{
	LPCTSTR ptr;
	int count = 0;
	bool error = false;
	int octet;

	if (ipaddr.length() > 15)
	{
		error = true;
	} else
	{
		ptr = (LPCTSTR) ipaddr;

		octet = -1;

		while (1)
		{
			if(isdigit(*ptr))
			{
				if (octet < 0) 
					octet = 0;
				octet = octet * 10 + (*ptr - '0');

			} else if (*ptr == '.')
			{
				// if ipaddr = ".1.23.1.4" || "1.2.3.4." then it is error
				if (octet < 0 || octet > 255)
				{
					error = true;
					break;
				} 
				_ipoctet[count] = (BYTE)octet;
				octet = -1;
				count++;

			} else if (*ptr == '\0')
			{
				if (octet < 0 || octet > 255 || count != 3)
				{
					error = true;
					break;
				} 
				_ipoctet[count] = (BYTE)octet;
				break;
			} else
			{		// characters other than digit and '.'
				error = true;
				break;
			}
			ptr++;
		}
	}

	if (error == true)
	{
		_ipoctet[0] = 0;
		_ipoctet[1] = 0;
		_ipoctet[2] = 0;
		_ipoctet[3] = 0;
	}
}

void
CIPAddr::SetIP(DWORD ipaddr, bool networkbyteorder)
{
	// Are we entering the number in network byte order?
	if (networkbyteorder == false)
	{
		_ipoctet[3] = (BYTE) (ipaddr & 0xFF);
		_ipoctet[2] = (BYTE) ((ipaddr >> 8) & 0xFF);
		_ipoctet[1] = (BYTE) ((ipaddr >> 16) & 0xFF);
		_ipoctet[0] = (BYTE) ((ipaddr >> 24) & 0xFF);

	} else
	{
		_ipoctet[0] = (BYTE) (ipaddr & 0xFF);
		_ipoctet[1] = (BYTE) ((ipaddr >> 8) & 0xFF);
		_ipoctet[2] = (BYTE) ((ipaddr >> 16) & 0xFF);
		_ipoctet[3] = (BYTE) ((ipaddr >> 24) & 0xFF);
	}
}

void
CIPAddr::SetIP(const CIPAddr &cipaddr)
{
	_ipoctet[0] = cipaddr[0];
	_ipoctet[1] = cipaddr[1];
	_ipoctet[2] = cipaddr[2];
	_ipoctet[3] = cipaddr[3];
}

void
CIPAddr::GetIP(CString &ipstr)
{
	TCHAR str[16];
	sprintf(str,_T("%d.%d.%d.%d"), _ipoctet[0], _ipoctet[1], _ipoctet[2], _ipoctet[3]);
	ipstr = str;
}

DWORD
CIPAddr::GetIP(bool networkbyteorder)
{
	DWORD _dwipaddr;

	if (networkbyteorder)
	{
		_dwipaddr = _ipoctet[3];
		_dwipaddr <<= 8;
		_dwipaddr |= _ipoctet[2];
		_dwipaddr <<= 8;
		_dwipaddr |= _ipoctet[1];
		_dwipaddr <<= 8;
		_dwipaddr |= _ipoctet[0];
	} else
	{
		_dwipaddr = _ipoctet[0];
		_dwipaddr <<= 8;
		_dwipaddr |= _ipoctet[1];
		_dwipaddr <<= 8;
		_dwipaddr |= _ipoctet[2];
		_dwipaddr <<= 8;
		_dwipaddr |= _ipoctet[3];
	}

	return _dwipaddr;
}

//	Typecast the CIPAddress to a CString()
CIPAddr::operator CString() const
{
	CString returnip;
	CString str_iptemp[4];
	TCHAR	sz_temp[255];

	// Convert each of the bytes into string number representation
	for (int loop = 0; loop < 4; loop++)
	{
		sprintf(sz_temp, "%d", (int)_ipoctet[loop]);
		//itoa((int)_ipoctet[loop], sz_temp, 10);
		str_iptemp[loop] = sz_temp;
	}

	// Build the cstring that will get returned.
	returnip = str_iptemp[0] + "." + str_iptemp[1] + "." + str_iptemp[2] + "." + str_iptemp[3];
	return returnip;
}

/*
 *	Typecast the CIPAddress to a DWORD - do it in network byte order.
 */
CIPAddr::operator DWORD()
{
	DWORD dw_return = 0;

	dw_return = _ipoctet[3];
	dw_return <<= 8;
	dw_return |= (DWORD)_ipoctet[2];
	dw_return <<= 8;
	dw_return |= (DWORD)_ipoctet[1];
	dw_return <<= 8;
	dw_return |=  (DWORD)_ipoctet[0];
	return dw_return;
}
CIPAddr & 
CIPAddr::operator=(const CIPAddr &cipaddr)
{
	_ipoctet[0] = cipaddr[0];
	_ipoctet[1] = cipaddr[1];
	_ipoctet[2] = cipaddr[2];
	_ipoctet[3] = cipaddr[3];

	return *this;
}

bool 
CIPAddr::operator== (const CIPAddr &cipaddr)
{
	return ((_ipoctet[0] == cipaddr[0]) && (_ipoctet[1] == cipaddr[1]) &&
				(_ipoctet[2] == cipaddr[2]) && (_ipoctet[3] == cipaddr[3]));
}

bool 
CIPAddr::operator!= (const CIPAddr &cipaddr)
{
	return ( ! ((_ipoctet[0] == cipaddr[0]) && (_ipoctet[1] == cipaddr[1]) &&
				(_ipoctet[2] == cipaddr[2]) && (_ipoctet[3] == cipaddr[3])) );
}

bool 
CIPAddr::operator<= (const CIPAddr &cipaddr)
{
	for (int i = 0; i < 4; i++)
	{
		if (_ipoctet[i] == cipaddr[i])
			continue;
		else if (_ipoctet[i] < cipaddr[i])
			return true;
		else 
			return false;
	}

	return true;
}

bool 
CIPAddr::operator>= (const CIPAddr &cipaddr)
{
	for (int i = 0; i < 4; i++) {
		if (_ipoctet[i] == cipaddr[i])
			continue;
		else if (_ipoctet[i] > cipaddr[i])
			return true;
		else
			return false;
	}

	return true;
}

bool 
CIPAddr::operator< (const CIPAddr &cipaddr)
{
	for (int i = 0; i < 4; i++) {
		if (_ipoctet[i] == cipaddr[i])
			continue;
		else if (_ipoctet[i] < cipaddr[i])
			return true;
		else 
			break;
	}

	return false;
}

bool 
CIPAddr::operator> (const CIPAddr &cipaddr)
{
	for (int i = 0; i < 4; i++) {
		if (_ipoctet[i] == cipaddr[i])
			continue;
		else if (_ipoctet[i] > cipaddr[i])
			return true;
		else
			break;
	}

	return false;
}

BYTE 
CIPAddr::operator[](int index) const
{
	// Return the ip number at the index
	return (_ipoctet[index]);
}

ostream& 
operator<< (ostream& os, CIPAddr &cip)
{
	CString strout;
	cip.GetIP(strout);
	os << strout;
	return os;
}

bool 
CIPAddr::IsNull()
{
	return (_ipoctet[0] == 0 && _ipoctet[1] == 0 && 
			_ipoctet[2] == 0 && _ipoctet[3] == 0);
}

bool 
CIPAddr::IsMulticast()
{
	return (_ipoctet[0] >= 224 && _ipoctet[0] < 240);
}

bool 
CIPAddr::IsBroadcast()
{
	return (_ipoctet[0] == 255 && _ipoctet[1] == 255 && 
			_ipoctet[2] == 255 && _ipoctet[3] == 255);
}

bool 
CIPAddr::IsLoopback()
{
	return (_ipoctet[0] == 127 && _ipoctet[1] == 0 && 
			_ipoctet[2] == 0   && _ipoctet[3] == 1);
}

bool 
CIPAddr::IsOnBoundary()
{
	for (int i = 0; i < 4; i++) 
		if (_ipoctet[i] == 0 || _ipoctet[i] == 255)
			return false;

	return true;
}
