
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include "pxemisc.h"
#include "cproxreg.h"
#include "cipaddr.h"

#include <sys/ioctl.h>
#include <sys/stat.h>
#include <net/if.h>
#include <syslog.h>

CProxyRegistry::CProxyRegistry()
{
	ConnectTo(HKEY_LOCAL_MACHINE, NULL);
}

CProxyRegistry::CProxyRegistry(LPCTSTR p_pkszMachine)
{
	ConnectTo(HKEY_LOCAL_MACHINE, p_pkszMachine);
}


// Return the name of the boot file for the specified architecture.
// Return NULL if none found.
// arch/Bootfile
BOOL CProxyRegistry::
GetBootFile (CString Arch, CString *p_pstrBootfile )
{
	CString path = PROXY_PATH + Arch;

	return(ReadRegString(path.data(), PROXREG_BOOTFILE, p_pstrBootfile));
}

// Return the name of the image file for the specified architecture.
// Return NULL if none found.
BOOL CProxyRegistry::
GetImageFile (USHORT p_usArch, BYTE *p_pucNic, USHORT p_usService,
				USHORT p_usLayer, BYTE* pCredentialType, int nCredLen,  CString *p_strImagePath, unsigned int *pSelectedCredType )
{
	CString strArch;
	CString strNic;
	CString strService;
	CString strRegPath = PROXY_PATH;
	CStringArray saImageFile;
	CString strImagePath;
	BOOL bCredentialsReqd = FALSE;
	 
	if (!GetArchPath(p_usArch, &strArch))
	{
		return FALSE;
	}

	if (!GetNicPath(p_pucNic, strArch, &strNic))
	{
		return FALSE;
	}
	
	if (!GetServiceName(p_usService, &strService))
	{
		return FALSE;
	}

	// PXE after PDK 2.1 uses the following line.
	strRegPath += strArch + CString(_T("/")) + strNic + CString(_T("/")) + strService;

	if (!ReadRegMultiString(strRegPath.data(), PROXREG_IMAGEFILE, &saImageFile))
	{
		return FALSE;
	}

	// Now see whether the layer field's upper bit is set, which indicates whether
	// credential file is being requested.

	if (pCredentialType != NULL)
		bCredentialsReqd = TRUE;

	// make sure that the upper bit is cleared. This bit may have been set if the 
	// the request is for credential types

	p_usLayer &= 0x7fff; // Mask out the upper bit
	// Check to see if there are anymore files left to send.
	int cnt=0;
	int nStartLayer=0, nEndLayer=0;
	char *pImageName = NULL;
	while(cnt < saImageFile.GetSize())
	{
		switch(cnt)
		{
		case 0:
			nStartLayer = atoi(saImageFile[0].data());
			break;
		case 1:
			nEndLayer = atoi(saImageFile[1].data());

			break;
		case 2:
			pImageName = (char*)(saImageFile[2].data());
			break;
		}

	       

		cnt++;
	}
	if ( !( (p_usLayer >= nStartLayer ) && (p_usLayer <= nEndLayer ) ))
	{
		return FALSE;
	}

	char pImagePath[1024];
	if (!bCredentialsReqd)
	{
		// PXE after PDK 2.1 uses the following line.
		sprintf(pImagePath, _T( "%s/%s/%s/%s.%d"),
				strArch.data(), strNic.data(), strService.data(), pImageName, p_usLayer);

		*p_strImagePath = pImagePath;
		return TRUE;
	}


	// Credentials required
	unsigned int nCredentialType;
	int idx = 0;
	unsigned char nBuf[4];
	while (idx < nCredLen)
	{
		char bufx[1024];
		// Get the base dir only once and not every time we enter this routine
		
		static BOOL bBasePathKnown = FALSE;
		static CString csBaseDir;
		if (!bBasePathKnown)
		{
			GetMtftpdRootDirectory(&csBaseDir);
			bBasePathKnown = TRUE;
		}

			
		memcpy(nBuf, pCredentialType+idx, 4); // copy 4 bytes at a time
		idx += 4; 
		nCredentialType = ntohl(*(unsigned int *)nBuf);


		sprintf(pImagePath, _T( "%s/%s/%s/%s.%d.cr%ld"),
							strArch.data(), strNic.data(), strService.data(), 
							pImageName, p_usLayer,	nCredentialType);

		CString csFullPath = csBaseDir + CString(_T("/")) + pImagePath;

		printf("BIS File name %s\n", csFullPath.data());		
		struct stat sbuf;
		
		if (stat(csFullPath.data(), &sbuf) != 0)
		{
			sprintf(bufx, "File %s not found\n", csFullPath.data());
			OutputDebugString(bufx);
			continue;
			// the file does not exist
		}
		
		
		// Verify whether this file exists.
		// Assume it does for the time being
		*p_strImagePath = pImagePath;
		*pSelectedCredType = nCredentialType;
		return TRUE;
	}

	return FALSE;
}

// Return the path of the Service Options DLL.
// Return FALSE if none found.
BOOL CProxyRegistry::
GetServiceOptionsDll (USHORT p_usArch, BYTE *p_pucNic, USHORT p_usService,
								   CString *p_strOptionsDll )
{
	CString strArch;
	CString strNic;
	CString strService;
	CString strRegPath = PROXY_PATH;
	CString strOptionsDll;
	 
	if (!GetArchPath(p_usArch, &strArch))
	{
		return FALSE;
	}

	if (!GetNicPath(p_pucNic, strArch, &strNic))
	{
		return FALSE;
	}
	
	if (!GetServiceName(p_usService, &strService))
	{
		return FALSE;
	}
	strRegPath += strArch + _T("/") + strNic + _T("/") + strService;
	if (!ReadRegString(strRegPath.data(), PROXREG_VENDOR_DLL, &strOptionsDll))
	{
		return FALSE;
	}

	*p_strOptionsDll = strOptionsDll;
	return TRUE;
}

//========================
// Return IP address of the Discovery Server that will provide BStrap, in network order.
// If no server is specified it will return the address of the local machine.
// Discovery_Srvr_IP
ULONG CProxyRegistry::
GetDiscoverySrvrIP()
{
	CString csSrvrIP;
	CIPAddr cipSrvrIP;

	if (ReadRegString(PROXY_PATH, PROXREG_DISCOVERY_SRVR_IP, &csSrvrIP))
	{
		cipSrvrIP.SetIP(csSrvrIP);
		return(cipSrvrIP.GetIP());
	} else
		return 0;
}

ULONG CProxyRegistry::
GetBroadcastIPAddress()
{
	CString csSrvrIP;
	CIPAddr cipSrvrIP;

	if (ReadRegString(PROXY_PATH, PROXREG_BROADCAST_IP_ADDRESS, &csSrvrIP))
	{
		cipSrvrIP.SetIP(csSrvrIP);
	}
	else
	{
		// assume all ones if this entry is not present
		cipSrvrIP.SetIP("255.255.255.255");
	}
	return(cipSrvrIP.GetIP());
}

// Return multi-cast discovery IP address for boot servers in network order.
// If none defined, return 0.
// Discovery_Addr

ULONG CProxyRegistry::
GetDiscoveryAddr()
{
	CString csSrvrAddr;
	CIPAddr cipSrvrAddr;

	if (ReadRegString(PROXY_PATH, PROXREG_DISCOVERY_MCAST_ADDR, &csSrvrAddr))
	{
		cipSrvrAddr.SetIP(csSrvrAddr);
		return(cipSrvrAddr.GetIP());
	} else
		return 0;
}


// Return BOOL for representation of the Discovery Broadcast Flag
// Returns FALSE if not defined.
// Discovery_BCast_Disbled
BOOL CProxyRegistry::
GetBcastFlag()
{
	DWORD Value;

	if (ReadRegDWord(PROXY_PATH, PROXREG_DISCOVERY_BCAST_DISABLED, &Value))
	{
		return (Value > 0);
	} else
		return FALSE;
}


// Return BOOL for value of the Multicast Disable flag
// Returns FALSE if not defined.
// Discovery_MCast_Disabled
BOOL CProxyRegistry::
GetMcastFlag()
{
	DWORD Value;

	if (ReadRegDWord(PROXY_PATH, PROXREG_DISCOVERY_MCAST_DISABLED, &Value))
	{
		return (Value > 0);
	} else
		return FALSE;
}

BOOL CProxyRegistry::
GetBStrapFlag()
{
	DWORD Value;

	if (ReadRegDWord(PROXY_PATH, PROXREG_LOCAL_BSTRAP, &Value))
	{
		return (Value > 0);
	} else
		return FALSE;
}


//---------------------------------------------------------------------------------
// Return BOOL for value of the Discovery Server List only flag
// Returns FALSE if not defined.
// Discovery_Server_List_Only
BOOL CProxyRegistry::
GetServerListFlag()
{
	DWORD Value;

	if (ReadRegDWord(PROXY_PATH, PROXREG_DISCOVERY_SERVER_LIST_ONLY, &Value))
	{
		return (Value > 0);
	} else
		return FALSE;
}

// Return Array of CStrings.  Each CString is an authorized server and its type
// Returns empty array if no servers are specified.
// Discovery_List
BOOL CProxyRegistry::
GetServerList(CStringArray *ServerList)
{
	return (ReadRegMultiString(PROXY_PATH, PROXREG_DISCOVERY_LIST, ServerList));
}

// Return a CStringArray with all of the Menus for the specified arch and NIC type.
// Returns empty array if no servers are specified.
// arch/nic/Menu
BOOL CProxyRegistry::
GetMenu (CString ArchPath, CString NicPath, CStringArray *Menu)
{
	CString Path = PROXY_PATH + ArchPath + _T("/") + NicPath;
	return (ReadRegMultiString(Path.data(), PROXREG_MENU, Menu));
}

// Return a CString with the prompt and display duration.
// Prompt
BOOL CProxyRegistry::
GetPrompt(CString *p_pszPrompt)
{
		return (ReadRegString(PROXY_PATH, PROXREG_PROMPT, p_pszPrompt));
}

// Translate an integer into an Text form and return the text in a CString.
// Arch_Text
BOOL CProxyRegistry::
GetArchPath(USHORT Arch, CString *p_szArch)
{
	CStringArray saArchs;

	if (ReadRegMultiString(PROXY_PATH, PROXREG_PROC_ARCH, &saArchs))
	{
		for (int i=0; i < saArchs.GetSize(); i++)
		{
			int nType, nPos; char archName[100];
			CString *cs = new CString();
			*cs = saArchs[i];
			nPos = cs->find(",");
			nType = atoi((cs->substr(0, nPos)).data());

			if (Arch == (USHORT)nType)
			{
				*p_szArch = cs->substr(nPos+1, cs->length());
				return TRUE;
			}
		}
	}
	return (FALSE);
}


// Match NIC type to the longest NIC type specified in the Arch Path provided.
// Return empty string if none found.
// Based on NIC sub keys defined in the ArchPath
#define UNDI_TYPE	0X1
#define UNDI_MAJ_VER	0X2
#define UNDI_MIN_VER	0X1


BOOL CProxyRegistry::
GetNicPath(BYTE *NicType, CString ArchPath, CString *p_pstrNicPath)
{
	CString Path = PROXY_PATH + ArchPath;
	CStringArray saNics;
	CString strNicType;
	int iMaxFound = 0;
	int idx = -1;


	if (NicType[0] == UNDI_TYPE &&
		NicType[1] == UNDI_MAJ_VER &&
		NicType[2] == UNDI_MIN_VER)
	{
		// This is a path representing UNDI
		*p_pstrNicPath = "UNDI";
		return TRUE;
	}
	else
	{
		*p_pstrNicPath = "UNDI";
		return TRUE;
	}
}

// Translate an integer into a Text form and return the text in a CString.
// Service_Types
BOOL CProxyRegistry::
GetServiceName(USHORT p_usService, CString *p_szServiceName)
{
	CStringArray saServices;

	if (ReadRegMultiString(PROXY_PATH, PROXREG_SERVICE_TYPES, &saServices))
	{
		for (int i=0; i < saServices.GetSize(); i++)
		{
			int nType, nPos; char archName[100];
			CString *cs = new CString();
			*cs = saServices[i];
			nPos = cs->find(",");
			nType = atoi((cs->substr(0, nPos)).data());
			if (p_usService == nType)
			{
				*p_szServiceName = cs->substr(nPos+1, cs->length());
				return TRUE;
			}
		}
	}
	return (FALSE);
}

// Return UseDHCPPort value.  Return 0 if not found.
// UseDHCPPort
BOOL CProxyRegistry::
UseDHCPPort()
{
	DWORD dwFlag = 0;
	if (ReadRegDWord(PROXY_PATH, PROXREG_USEDHCPPORT, &dwFlag))
	{
		return (BOOL) dwFlag;
	} else
		return FALSE;
}


// Read Debug value.  Return True if set.
// Debug
BOOL CProxyRegistry::
GetDebugFlag()
{
	DWORD dwFlag = 0;
	if (ReadRegDWord(PROXY_PATH, PROXREG_DEBUG, &dwFlag))
	{
		return (BOOL) dwFlag;
	} else
		return FALSE;
}


// Read TestOn value.  Return True if set
// TestOn
BOOL CProxyRegistry::
TestOn()
{
	DWORD dwFlag = 0;
	if (ReadRegDWord(PROXY_PATH, PROXREG_TESTON, &dwFlag))
	{
		return (BOOL) dwFlag;
	} else
		return FALSE;
}

// Read TestPath value.  Return True if set
// TestPath
BOOL CProxyRegistry::
TestPath(CString& p_strTestPath)
{
	CString strTestPath;

	if (ReadRegString(PROXY_PATH, PROXREG_TESTPATH, &strTestPath))
	{
		if (strTestPath.length() != 0)
		{
			p_strTestPath = strTestPath;
		}
	} 
	else
	{
		p_strTestPath = "/tmp";
	}

	return TRUE;
}

// Read StartProxy value. Return TRUE if set.
// StartProxy
BOOL CProxyRegistry::
StartProxyFlag()
{
	DWORD dwFlag = 0;
	if (ReadRegDWord(PROXY_PATH, PROXREG_STARTPROXY, &dwFlag))
	{
		return (BOOL) dwFlag;
	} else
		return FALSE;
}


// Read StartProxy value. Return TRUE if set.
// StartBinl
BOOL CProxyRegistry::
StartBinlFlag()
{
	DWORD dwFlag = 0;
	if (ReadRegDWord(PROXY_PATH, PROXREG_STARTBINL, &dwFlag))
	{
		return (BOOL) dwFlag;
	} else
		return FALSE;
}

// Read StartBootService value. Return TRUE if set.
// StartBootService
BOOL CProxyRegistry::
StartBootServiceFlag()
{
	DWORD dwFlag = 0;
	if (ReadRegDWord(PROXY_PATH, PROXREG_STARTBOOTSERVICE, &dwFlag))
	{
		return (BOOL) dwFlag;
	} else
		return FALSE;
}

// Read MasterProxy value. Return TRUE if set.
// MasterProxy
BOOL  CProxyRegistry::
MasterProxyFlag()
{
	DWORD dwFlag = 0;
	if (ReadRegDWord(PROXY_PATH, PROXREG_MASTERPROXY, &dwFlag))
	{
		return (BOOL) dwFlag;
	} else
		return FALSE;
}

// Read/Set Server Name, Domain Name, and IsDomain flag.
BOOL CProxyRegistry::
GetMachineInfo(CString* lpcsMachine, CString* lpcsLanGrp, BOOL &bIsDomain)
{
	CString csPath = PROXY_PATH;
	CString csField = PROXREG_SERVER_NAME;
	BOOL bResult = FALSE;

	bResult = ReadRegString(csPath, csField, lpcsMachine);
	if (!bResult)
		return FALSE;

	csField = PROXREG_DOMAIN;
	bResult = ReadRegString(csPath, csField, lpcsLanGrp);
	if (!bResult)
		return FALSE;

	csField = PROXREG_ISDOMAIN;
	DWORD dwIsDomain = 0;
	bResult = ReadRegDWord(csPath, csField, &dwIsDomain);
	if (!bResult)
		return FALSE;
	else
	{
		bIsDomain = (dwIsDomain != 0);
		return TRUE;
	}

}



// This function was put in to allow testing with delays in the transmission
// and is not used in production code.
DWORD CProxyRegistry::
GetDelay()
{
	DWORD dwDelay = 0;
	if (ReadRegDWord(PROXY_PATH, PROXREG_DELAY, &dwDelay))
	{
		return (BOOL) dwDelay;
	} else
		return 0;

}

ULONG CProxyRegistry::
GetMipBase()
{
	//printf("GetMipBase not implemented\n");
	return 0;
/*
	CString strData;

	if(ReadRegString(PROXY_PATH, PROXREG_MCAST_BASE_ADDRESS, &strData))
	{
		CIPAddr cipData(strData);
		return(cipData.GetIP());
	} else
		return 0;
*/
}

ULONG CProxyRegistry::
GetMipLast()
{
	//printf("GetMipLast not implemented\n");
	return 0;
/*
	CString strData;

	if(ReadRegString(PROXY_PATH, PROXREG_MCAST_LAST_ADDRESS, &strData))
	{
		CIPAddr cipData(strData);
		return(cipData.GetIP());
	} else
		return 0;
*/
}

USHORT CProxyRegistry::
GetMipBlock()
{
	//printf("GetMipBlock not implemented\n");
	return 0;
/*
	CString strData;

	if(ReadRegString(PROXY_PATH, PROXREG_MCAST_BLOCK_SIZE, &strData))
	{
		return (USHORT) _ttoi((LPCTSTR)strData);

	} else
		return 0;
*/
}

USHORT CProxyRegistry::
GetMipRange()
{
	//printf("GetMipRange not implemented\n");
	return 0;
/*
	CString strData;

	if(ReadRegString(PROXY_PATH, PROXREG_MCAST_ADDRESS_RANGE, &strData))
	{
		return (USHORT) _ttoi((LPCTSTR)strData);
	} else
		return 0;
*/
}

BOOL  CProxyRegistry ::GetSystemIPAddress(CString* pData)
{
	struct sockaddr_in sin;
	struct ifreq ifr;
	int skfd;
	CString ifname;
	static CString csIp;
	static int bIpAddressKnown = FALSE;

	if (bIpAddressKnown == TRUE)
	{
		*pData = csIp;
		return TRUE;
	}
	GetInterfaceName(&ifname);
	while (1)
	{
		char *pp;
		skfd = socket(AF_INET, SOCK_DGRAM, 0);
		if (skfd != -1)
		{
			strcpy(ifr.ifr_name, ifname.data()); 
		
			if (ioctl(skfd, SIOCGIFADDR, &ifr) < 0) 
			{
				syslog(LOG_ERR, "IOCTL SIOCGIFADDR failed for interface %s\n", ifname.data());
				break;
			} 
			else 
			{
				memcpy(&sin, &(ifr.ifr_addr), sizeof(struct sockaddr));
				struct in_addr  inn ;
				inn.s_addr =  sin.sin_addr.s_addr;
				pp = inet_ntoa(inn);	
				*pData = pp;
				csIp = *pData;
				syslog(LOG_INFO, "PXE server will be using IP address %s\n", csIp.data());
				bIpAddressKnown = TRUE;
				close(skfd);
				return TRUE;
			}
		}

	}
	close(skfd);

	if (ReadRegString(PROXY_PATH, PROXREG_IP_ADDRESS, pData))
	{
		syslog(LOG_INFO, "PXE server uses IP address %s from pxe.conf file\n", pData->data());
		return (TRUE);
	}
	else
	{
		syslog(LOG_ERR, "PXE server cannot determine its own IP address from pxe.conf file\n");
		exit(1);
		return FALSE;
	}
}


BOOL CProxyRegistry::GetMtftpdRootDirectory (CString *p_path)
{
	return (ReadRegString(PROXY_PATH, PROXREG_MTFTPD_ROOT_DIR, p_path));
}


BOOL CProxyRegistry::GetInterfaceName (CString *p_path)
{
	if (ReadRegString(PROXY_PATH, PROXREG_INTERFACE_NAME, p_path))
		return TRUE;

	// Assume a default of "eth0"
	*p_path = "eth0";
	return TRUE;
}

UINT CProxyRegistry ::
GetClients(CStringArray* p_pstrarClients)
{
	ReadRegMultiString(PROXY_PATH, PROXREG_PARSERS, p_pstrarClients);
	return(p_pstrarClients->GetSize());
}

BOOL  CProxyRegistry ::
GetParserFunction(CString p_csParser, CString* p_csDllPath, CString* p_csFuncName)
{
	CString csPath =  PROXY_PATH;
	csPath += PROXREG_PARSERS;
	csPath += _T("/");
	csPath += p_csParser;

	if (ReadRegString(csPath, PROXREG_DLLPATH, p_csDllPath))
	{
		if (ReadRegString(csPath, PROXREG_MAINFUNCTION, p_csFuncName))
		{
			return TRUE;
		}
	}
	return FALSE;
}


