
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include <stdarg.h>
#include "pxemisc.h"
#include "cpxedebug.h"
#include <syslog.h>


CPxeDebug::CPxeDebug()
{
	m_nType = 0;
	m_pReg = NULL;
	m_fp = NULL;
	bDebugOutToWindow = FALSE;
	bDebugOutToFile = FALSE;
}

BOOL CPxeDebug::init(int nType)
{

	CString csODSFlag, csDebugFileFlag, csDebugFileName, csDefaultFileName, csFileName;
	m_nType = nType;
	m_pReg = new CRegistry();
	m_pReg->ConnectTo(HKEY_LOCAL_MACHINE);

	switch (nType)
	{
		case PXE_PARSER_DEBUG:

			csODSFlag =	ODS_PARSERS_ON;
			csDebugFileFlag = DEBUG_PARSERS_ON;
			csDebugFileName = DEBUG_PARSERS_FILENAME;
			csDefaultFileName = DEFAULT_PARSERS_LOGFILE;

			break;

		case PXE_SERVICES_DEBUG:

			csODSFlag = ODS_SERVICES_ON;
			csDebugFileFlag = DEBUG_SERVICES_ON;
			csDebugFileName = DEBUG_SERVICES_FILENAME;
			csDefaultFileName = DEFAULT_SERVICES_LOGFILE;

			break;

		case PXE_TESTER_DEBUG:

			csODSFlag = ODS_TESTER_ON;
			csDebugFileFlag = DEBUG_TESTER_ON;
			csDebugFileName = DEBUG_TESTER_FILENAME;
			csDefaultFileName = DEFAULT_TESTER_LOGFILE;

			break;

		case PXE_MTFTP_DEBUG:

			csODSFlag = ODS_MTFTP_ON;
			csDebugFileFlag = DEBUG_MTFTP_ON;
			csDebugFileName = DEBUG_MTFTP_FILENAME;
			csDefaultFileName = DEFAULT_MTFTP_LOGFILE;

			break;
		default:
			break;

	}

	BOOL bSuccess;
	bSuccess = m_pReg->ReadRegDWord(DEBUG_ROOT_PATH, csODSFlag, (DWORD*)&bDebugOutToWindow);

	bSuccess = m_pReg->ReadRegDWord(DEBUG_ROOT_PATH, csDebugFileFlag, (DWORD*)&bDebugOutToFile);
	bSuccess = m_pReg->ReadRegString(DEBUG_ROOT_PATH, csDebugFileName, &csFileName);
	if (!bSuccess)
	{
		csFileName = csDefaultFileName;
	}

	if (bDebugOutToFile)
	{
		// Get the name of the file first or assume our own file name
		m_fp = fopen(csFileName, "w");
		if (m_fp == NULL)
		{
			// Set the flag to false as we cannot write to the file
			bDebugOutToFile = FALSE;
			return FALSE;
		}
	}

	return TRUE;
}

BOOL CPxeDebug::close()
{
	if (m_pReg)
		delete m_pReg;
	if (m_fp)
		fclose(m_fp);
	return TRUE;
}

int CPxeDebug::write(char *lpszFormat, ... )
{
	if (!DebugOn())
		return 0;

	va_list args;
	va_start(args, lpszFormat);

	int nBuf;
	TCHAR szBuffer[1024];

	nBuf = vsnprintf(szBuffer, sizeof(szBuffer), lpszFormat, args);

	va_end(args);
	
	if (m_fp && bDebugOutToFile)
	{
		printf("writing to file..\n");
		fprintf(m_fp, szBuffer);
		fflush(m_fp);
	}

	if (bDebugOutToWindow)
	{
		OutputDebugString(szBuffer);
	}
	return 1;
}

int CPxeDebug::logError(char *lpszFormat, ... )
{
	va_list args;
	va_start(args, lpszFormat);

	int nBuf;
	TCHAR szBuffer[1024];

	nBuf = vsnprintf(szBuffer, sizeof(szBuffer), lpszFormat, args);

	va_end(args);

	// Write to the our own error log file irrespective of what the debug flag says
	if (m_fp)
	{
		fprintf(m_fp, szBuffer);
	}
	openlog("PXE", LOG_PID, LOG_DAEMON);
	syslog(LOG_ERR, szBuffer);
	closelog();

	return 1;
}

int CPxeDebug::logError(int nMsgId, char *lpszFormat, ... )
{

	va_list args;
	va_start(args, lpszFormat);

	int nBuf;
	TCHAR szBuffer[1024];

	nBuf = vsnprintf(szBuffer, sizeof(szBuffer), lpszFormat, args);

	va_end(args);

	// Write to the our own error log file irrespective of what the debug flag says
	if (m_fp)
	{
		fprintf(m_fp, szBuffer);
	}
	openlog("PXE", LOG_PID, LOG_DAEMON);
	syslog(LOG_ERR, szBuffer);
	closelog();

	return 1;
}


void CPxeDebug::HexDump(unsigned char *buf, int nSize)
{
	// Not implemented for Linux
	if (m_fp && bDebugOutToFile)
	{
		//::fHexDump(m_fp, buf, nSize);
	}
}
	
BOOL CPxeDebug::DebugOn()
{
	// Should return TRUE if either OutputDebugString or Writing to a file is required
	return (bDebugOutToWindow || bDebugOutToFile) ;
}
