
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include "pxemisc.h"
#include "registry.h"

static int siInstance = 0;


//
// In this current implementation, registry is implemented as a flat file,
// with sections. It is very similar to the .ini files present in windows 3.1
// systems.
//
//
// Macros for UNICODE implementation.
#define SIZEOFCHAR (sizeof(TCHAR))
#define SIZEOFNULL (sizeof(TCHAR))

#define READ_SIZE	1024
// remember Validate Handle using some API in every call
CRegistry::CRegistry ()
{
	m_bIsValid = FALSE;		// Cueerntly The m_hRegKey is not valid

}

CRegistry::~CRegistry ()
{

}

BOOL CRegistry::
IsConnected()
{
	return m_bIsValid;
}

BOOL CRegistry::ConnectTo (HKEY p_hKey, const TCHAR *p_pkszMachine /* = NULL */) 
{
	// We will assume that we are always connected.
	m_bIsValid = TRUE;
		return TRUE;
}

// get the handle whenever this class cannot do what you want.
// please dont modify this handle.
BOOL CRegistry::GetHandleToHive (HKEY *p_phKey)
{
	
	return TRUE;
}

  
// ==================================================================================
// Some Global Functions for registry.
// ==================================================================================

#define	REG_STRING_LEN		200

BOOL CRegistry::ReadRegString ( LPCTSTR p_lpPath, 
					LPCTSTR p_lpField, 
					CString *p_pstrData) const
{
	FILE* fp;
	fp = OpenRegistry();
	if (fp == NULL)
		return FALSE;

	if (!FindRegEntry(fp, p_lpPath, p_lpField))
	{
		fclose(fp);
		return FALSE;
	}
	int ret = ReadStringValue(fp, p_pstrData);
	fclose(fp);
	return (ret)?TRUE:FALSE;
}


// Intial guess at length.
#define	REG_MULTISTRING_LEN		(512*sizeof(TCHAR))


BOOL CRegistry::ReadRegMultiString ( LPCTSTR p_lpPath, 
					LPCTSTR p_lpField, 
					CStringArray *p_pkroarrStrings) const
{
	FILE* fp;
	fp = OpenRegistry();
	if (fp == NULL)
		return FALSE;

	if (!FindRegEntry(fp, p_lpPath, p_lpField))
	{
		fclose(fp);
		return FALSE;
	}
	int ret = ReadMultiStringValues(fp, p_pkroarrStrings);
	fclose(fp);
	return (ret)?TRUE:FALSE;

}


BOOL CRegistry::ReadRegDWord ( LPCTSTR p_lpPath, 
					 LPCTSTR p_lpField,  
					 DWORD *p_pdwVal) const
{
	FILE* fp;
	fp = OpenRegistry();
	if (fp == NULL)
		return FALSE;

	if (!FindRegEntry(fp, p_lpPath, p_lpField))
	{
		fclose(fp);
		return FALSE;
	}
	int ret = ReadDwordValue(fp, p_pdwVal);
	fclose(fp);
	return (ret)?TRUE:FALSE;

}


// OpenRegistry() routine will access the reg file pxe.conf 
// under /etc directory. I am not happy to hardcode the 
// path of this file. However, I am not familiar with any other
// method under Linux. SO, it will remain this way till someone does it better

FILE *CRegistry::OpenRegistry() const
{

	return fopen("/etc/pxe.conf", "r");
}

// The intermediate entries in the registry are enclosed within
// [] at the start of a line. The leaf nodes (the actual config values)
// are present on new lines underneath each intermediate node.
//
// The entries are NOT case sensitive.

BOOL CRegistry::FindRegEntry(FILE *fp, LPCTSTR p_lpPath, LPCTSTR p_lpField) const
{
	char buf[READ_SIZE];

	char strToFind [READ_SIZE];

	strcpy(strToFind, "[");
	if (p_lpPath[0] != 0)
	{
		strcat(strToFind, p_lpPath);
		strcat(strToFind, "/");
	}

	strcat(strToFind, p_lpField);

	int nLen = strlen(strToFind);
	while(1)
	{
		fgets(buf, sizeof(buf), fp);
		if (feof(fp))
			break;
		if (strncasecmp(buf, strToFind,  nLen) == 0)
		{
			return TRUE;
		}

	}
	return FALSE;
}

BOOL CRegistry::ReadDwordValue(FILE *fp, DWORD* p_pdwVal) const
{
	char buf[READ_SIZE];

	buf[0] = 0; // initialize to NULL

	if (!ReadNextValueLine(fp, buf))
		return FALSE;

	*p_pdwVal = atoi(buf);
	return TRUE;
}

BOOL CRegistry::ReadStringValue(FILE *fp, CString* p_pstrData) const
{
	char buf[READ_SIZE];

	buf[0] = 0; // initialize to NULL

	if (!ReadNextValueLine(fp, buf))
		return FALSE;

	*p_pstrData = buf;
	return TRUE;
}

BOOL CRegistry::ReadMultiStringValues(FILE *fp, CStringArray* pArray) const
{
	char buf[READ_SIZE];
	BOOL bPresent = FALSE;

	buf[0] = 0; // initialize to NULL

	while (ReadNextValueLine(fp, buf))
	{
		bPresent = TRUE;
		pArray->Add(buf);
		buf[0] = 0;
	}

	return (bPresent)?TRUE:FALSE;
}

BOOL CRegistry::ReadNextValueLine(FILE* fp, char* buf) const
{

	buf[0] = 0; // initialize to NULL

	while(1)
	{
		fgets(buf, READ_SIZE, fp);
		if (feof(fp))
			break;
		int nRead = strlen(buf);
		nRead--; // Account for the newline char which is part of the read
		if (nRead <= 0) // skip blank lines
			continue; 

		buf[nRead] = 0; // Remove the \n from the buffer

		if (buf[0] == '[')
		{
			// another registry entry has started. so bail out
			break;
		}
		if (buf[0] == '#')
		{
			// it is a comment line, so continue
			continue;
		}
		else
		{
			return TRUE;
		}
	}
	return FALSE;
}
