
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */


#ifndef __CIPADDR_H_
#define __CIPADDR_H_

#include "pxemisc.h"
#include <iostream.h>

class CIPAddr {

protected: 
	// each octet stored seperately (_ipoctet[0] being the most 
	// significant octet)
	BYTE _ipoctet[4];	

public:
	CIPAddr();
	CIPAddr(const CString &ipaddr);
	CIPAddr(DWORD ipaddr, bool networkbyteorder = true);
	CIPAddr(const CIPAddr &cipaddr);

	CIPAddr & operator=(const CIPAddr &cipaddr);

	void SetIP(const CString &ipaddr);
	void SetIP(DWORD ipaddr, bool networkbyteorder = true);
	void SetIP(const CIPAddr &cipaddr);

	// convert IP address to string
	void GetIP(CString &str);
	// convert IP address to DWORD (in Little-Endian by default
	// else Big-Endian)
	DWORD GetIP(bool networkbyteorder = true);

	bool IsNull();
	bool IsMulticast();
	bool IsBroadcast();
	bool IsLoopback();

	// return true if any of the octets in the IP address is either
	// 0 or 255
	virtual bool IsOnBoundary();

	operator CString() const;
	operator DWORD();

	bool operator==(const CIPAddr &cipaddr);
	bool operator!=(const CIPAddr &cipaddr);

	bool operator>=(const CIPAddr &cipaddr);
	bool operator<=(const CIPAddr &cipaddr);
	bool operator>(const CIPAddr &cipaddr);
	bool operator<(const CIPAddr &cipaddr);

	BYTE operator[](int index) const;

	friend ostream& operator<< (ostream& os, CIPAddr& cipaddr);
};

#endif
