
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#ifndef _CPROXYREGISTRY_H_
#define _CPROXYREGISTRY_H_

#include "registry.h"
#define HKEY_LOCAL_MACHINE 0
#define PROXY_PATH	_T("")
#define PROXREG_BOOTFILE	_T("Bootfile")
#define PROXREG_CONTROL_TFTP	_T("Control_TFTP")
#define PROXREG_DEBUG _T("Debug")
#define PROXREG_DELAY _T("Delay")
#define PROXREG_DISCOVERY_BCAST_DISABLED _T("Discovery_BCast_Disabled")
#define PROXREG_BROADCAST_IP_ADDRESS	_T("Broadcast_IP_Address")
#define PROXREG_IP_ADDRESS	_T("OurIpAddress")
#define PROXREG_DISCOVERY_LIST _T("Discovery_List")
#define PROXREG_DISCOVERY_MCAST_ADDR _T("Discovery_MCast_Addr")
#define PROXREG_DISCOVERY_MCAST_DISABLED _T("Discovery_MCast_Disabled")
#define PROXREG_DISCOVERY_SERVER_LIST_ONLY _T("Discovery_Server_List_Only")
#define PROXREG_DISCOVERY_SRVR_IP	_T("Discovery_Srvr_IP")
#define PROXREG_DLLPATH _T("DllPath")
#define PROXREG_DOMAIN _T("Domain")
#define PROXREG_IMAGEFILE	_T("ImageFile_Name")
#define PROXREG_ISDOMAIN	_T("IsDomain")
#define PROXREG_LOCAL_BSTRAP	_T("Discovery_Local_Bstrap")
#define PROXREG_MASTERPROXY _T("MasterProxy")
#define PROXREG_MAINFUNCTION _T("MainFunction")
#define PROXREG_MCAST_ADDRESS_RANGE _T("Mcast_Address_Range")
#define PROXREG_MCAST_BASE_ADDRESS _T("Mcast_Base_Address")
#define PROXREG_MCAST_BLOCK_SIZE _T("Mcast_Block_Size")
#define PROXREG_MCAST_LAST_ADDRESS _T("Mcast_Last_Address")
#define PROXREG_MENU _T("MENU")
#define PROXREG_PARSERS _T("Parsers")
#define PROXREG_PROC_ARCH _T("PROC_ARCH")
#define PROXREG_PROMPT _T("Prompt")
#define PROXREG_SERVER_NAME _T("ServerName")
#define PROXREG_SERVICE_TYPES _T("Service_Types")
#define PROXREG_STARTBINL _T("StartBinl")
#define PROXREG_STARTBOOTSERVICE _T("StartBootService")
#define PROXREG_STARTPROXY _T("StartProxy")
#define PROXREG_TESTON _T("TestOn")
#define PROXREG_TESTPATH _T("TestPath")
#define PROXREG_USEDHCPPORT _T("UseDHCPPort")
#define PROXREG_VENDOR_DLL	_T("Vendor_DLL")
#define PROXREG_VENDOR_OPTIONS	_T("Vendor_Options")
#define PROXREG_CREDENTIALS_SUPPORTED	_T("Credential_Types_Supported")

#define PROXREG_MTFTPD_ROOT_DIR		_T("Mtftpd_Root_Directory")
#define PROXREG_INTERFACE_NAME		_T("Network_Interface_Name")

class CProxyRegistry : public CRegistry 
{
private:

public:

	CProxyRegistry();
	CProxyRegistry(LPCTSTR p_pkszMachine) ;

	// Return the name of the boot file for the specified architecture.
	// Return NULL if none found.
	BOOL GetBootFile (CString Arch, CString *p_pstrBootfile );

	// Get/Set IP address of the Discovery Server Reg Value in network order.
	// This value is primarily used to disambiguate the cases where there are multiple
	// IP addresses assigned to one machine.
	ULONG GetDiscoverySrvrIP();

	// Get Multi-cast discover IP Address for the boot server.
	// Multi-cast discovery IP address for boot servers returned in network order.
	// Input IP address is in network order.
	// If none defined, return 0.
	ULONG GetDiscoveryAddr();

	// Get BOOL for representation of the Discovery Broadcast Flag
	// Returns FALSE if not defined.
	BOOL GetBcastFlag();

	// Get BOOL for representation of the Force BStrap Flag
	// Returns FALSE if not defined.
	BOOL GetBStrapFlag();

	// Get BOOL value of the Multicast Disable flag
	// Returns FALSE if not defined.
	BOOL GetMcastFlag();

	// Get BOOL for value of the Discovery Server List Only flag
	// Returns FALSE if not defined.
	BOOL GetServerListFlag();

	// Get List of valid boot servers.
	//  Get returns an Array of CStrings.  Each CString is an authorized server and its type;
	// returns empty array if no servers are specified.
	// Set takes an array of CStrings; Each CString is an authorized server and its type.
	BOOL GetServerList(CStringArray *ServerList);

	// Get Menu List
	// Get returns a CStringArray with all of the Menus for the specified
	// architecture and NIC type. Returns empty array if no servers are specified.
	BOOL GetMenu (CString ArchPath, CString NicPath, CStringArray *Menu);

	// GET Prompt.
	// Return a CString with display duration and prompt separated by a comma.
	// Input parameter is same format.
	BOOL GetPrompt(CString *p_pszPrompt);

	// Translate an integer into an Text form and return the text in a CString.
	// Intel\ProxyDHCP\Arch_Text
	BOOL GetArchPath(USHORT Arch, CString *p_szArch);

	// Match NIC type to the longest NIC type specified in the Arch Path provided.
	// Return empty string if none found.
	// Based on NIC sub keys defined in the ArchPath
	//BOOL GetNicPath(CByteArray &NicType, CString ArchPath, CString *p_pstrNicPath);
	BOOL GetNicPath(BYTE *NicType, CString ArchPath, CString *p_pstrNicPath);
	//BOOL AddNicType(CString& p_csArch, CString& p_csNic);
	//BOOL DelNicType(CString& p_csArch, CString& p_csNic);

	// Return NULL if none found.
	// Intel\ProxyDHCP\arch\Bootfile
	BOOL GetImageFile (USHORT p_usArch, BYTE *p_pucNic, USHORT p_usService,
				USHORT p_usLayer, BYTE* pCredentialType, int nCredLen, 
				CString *p_strImagePath, unsigned int *pSelectedCredType  );


	// Return the path of the Service Options DLL.
	// Return FALSE if none found.
	BOOL GetServiceOptionsDll (USHORT p_usArch, BYTE *p_pucNic,
												USHORT p_usService, CString *p_strOptionsDll );

	// Translate an integer into a Text form and return the text in a CString.
	// Intel\ProxyDHCP\Service_Types
	BOOL GetServiceName(USHORT p_usService, CString *p_szServiceName);

	// Return UseDHCPPort value.  Return 0 if not found.
	// Intel/ProxyDHCP/UseDHCPPort
	BOOL UseDHCPPort();

	// Read Debug value.  Return True if set.
	// Intel\ProxyDHCP\Debug
	BOOL GetDebugFlag();

	// Read TestOn value.  Return True if set
	// Intel\PXE\ProxyDHCP\TestOn
	BOOL TestOn();

	BOOL TestPath(CString& p_strTestPath);

	// Read StartProxy value. Return TRUE if set.
	// Intel\PXE\ProxyDHCP\StartProxy
	BOOL StartProxyFlag();

	// Read StartProxy value. Return TRUE if set.
	// Intel\PXE\ProxyDHCP\StartBinl
	BOOL StartBinlFlag();

	// Read StartBootService value. Return TRUE if set.
	// Intel\PXE\ProxyDHCP\StartBootService
	BOOL StartBootServiceFlag();

	// Read StartBootService value. Return TRUE if set.
	// Intel\PXE\ProxyDHCP\StartBootService
	BOOL MasterProxyFlag();

	// Read/Set Server Name, Domain Name, and IsDomain flag.
	BOOL GetMachineInfo(CString* lpcsMachine, CString* lpcsLanGrp, BOOL &bIsDomain);

	// Read a delay time for testing late BINL responses.
	DWORD GetDelay();

	ULONG GetMipBase();
	//BOOL SetMipBase(ULONG usData);

	ULONG GetMipLast();
	//BOOL SetMipLast(ULONG usData);

	USHORT GetMipBlock();
	//BOOL SetMipBlock(USHORT usData);

	USHORT GetMipRange();
	//BOOL SetMipRange(USHORT usData);

	DWORD GetBroadcastIPAddress();
	DWORD GetSystemIPAddress(CString* pData);

	// Get the base/root directory of the MTFTPD files

	BOOL GetMtftpdRootDirectory (CString *p_path);
	BOOL GetInterfaceName (CString *p_path);

	UINT GetClients(CStringArray* p_pstrarClients);

	BOOL GetParserFunction(CString p_csParser, CString* p_csDllPath, CString* p_csFuncName);

};
#endif
