
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#ifndef	__PACKETDATA_H
#define __PACKETDATA_H

#include "dhcp.h"

#define GUID_LENGTH 16
#ifndef MAXDISCLEN
#define MAXDISCLEN	1500
#endif

enum PacketTypes {NULL_PKT, LSA1_CLIENT_PKT, LSA2V10_PROXY_PKT, LSA2V10_BINL_PKT,
						LCM_DISCOVER_PKT, PXE_SERVER_PKT, UNKNOWN_PKT};
enum ClientTypes {NULL_CLIENT, LSA1_CLIENT, LSA2V10_CLIENT, DISCOVERY_CLIENT, PXE_SERVER,
				UNKNOWN_CLIENT};
enum NICTypes {NT_UNDI = 1 , NT_PCI = 2 , NT_PNP = 3 };
extern FILE *fpDebug;


class CPacketData
{
private:

	ClientTypes		m_ClientType;
	BYTE			m_ucMsgType;
	USHORT			m_usNICID;
	BYTE			m_pucNICInterface[50];
	USHORT			m_usProcArch;
	BOOL			m_bNeedGUID;
	BOOL			m_bNeedProcArch;
	BOOL			m_bNeedNicID;
	BYTE			m_pucGUID[GUID_LENGTH];
	USHORT			m_usServiceType;
	BYTE			m_ucServiceLayer;

public:	// These are public for convience and should be changed to private.
	SOCKADDR_IN toSockAddr;
	char SendBuf[MAXDISCLEN];
	int numSent;
	int hEvent;
	int Mode;
	USHORT usReceivePort;

public:
	CPacketData()
	{
		m_ClientType = NULL_CLIENT;
		memset(m_pucNICInterface, '\0', sizeof(m_pucNICInterface));
		memset(m_pucGUID, '\0', GUID_LENGTH);
		m_bNeedGUID = TRUE;
		m_bNeedProcArch = TRUE;
		m_bNeedNicID = TRUE;
		fflush(NULL);
	};


	~CPacketData()
	{
	};

	void ClearPacketData()
	{
		m_ClientType = NULL_CLIENT;
		memset(m_pucNICInterface, '\0', sizeof(m_pucNICInterface));
		memset(m_pucGUID, '\0', GUID_LENGTH);
		m_bNeedGUID = TRUE;
		m_bNeedProcArch = TRUE;
		m_bNeedNicID = TRUE;
		fflush(NULL);
	};

	void ServiceType(USHORT p_usServiceType){m_usServiceType = p_usServiceType;};

	USHORT GetServiceType(){return m_usServiceType;};

	void ServiceLayer(BYTE p_ucServiceLayer){m_ucServiceLayer = p_ucServiceLayer;};

	BYTE GetServiceLayer() {return m_ucServiceLayer;};

	void MsgType( const BYTE ucMsgType) { m_ucMsgType = ucMsgType;};

	BYTE GetMsgType() { return m_ucMsgType; };

	void ClientType( const ClientTypes ct)
	{ 
		m_ClientType = ct;
	};

	ClientTypes ClientType()
	{ 
		return(m_ClientType);
	};

	ClientTypes GetClientType()
	{ 
		return(m_ClientType);
	};

	PacketTypes PacketType()
	{

		switch (m_ClientType)
		{
		case PXE_SERVER:
			return (PXE_SERVER_PKT);

		case DISCOVERY_CLIENT :
			return LCM_DISCOVER_PKT;
			break;

		case  LSA1_CLIENT :
			if ( (m_ucMsgType == DHCP_DISCOVER)
				&& (m_usNICID != 0) )
			{
				return (LSA1_CLIENT_PKT);

			} else
			{
				return UNKNOWN_PKT;
			}
			break;
 
		case LSA2V10_CLIENT :
			if ( (m_ucMsgType == DHCP_DISCOVER)
				&& (FALSE == m_bNeedProcArch)
				&& (FALSE == m_bNeedNicID) )
			{
				return (LSA2V10_PROXY_PKT);
			} else if ( (m_ucMsgType == DHCP_REQUEST)
				&& (FALSE == m_bNeedProcArch)
				&& (FALSE == m_bNeedNicID) )
			{
				return (LSA2V10_BINL_PKT);
			} else
			{
				return UNKNOWN_PKT;
			}
			break;

		default :
			return UNKNOWN_PKT;
		}
	};

	void NICId(const USHORT usNICID)
	{
		m_usNICID = usNICID;

	};

 	USHORT GetNICID(){return m_usNICID;};

	void NICInterface(const BYTE* pucNI,
						const BYTE ucLen)
	{
		m_pucNICInterface[0] = ucLen;
		memcpy(&m_pucNICInterface[1], pucNI, ucLen);
		m_bNeedNicID = FALSE;
	};

	BYTE GetNICInterface(BYTE* pucNI)
	{
		memcpy(pucNI, &m_pucNICInterface[1], m_pucNICInterface[0]);
		return (BYTE) m_pucNICInterface[0];
	}

	void ProcArch(const USHORT usProcArch)
	{
		m_usProcArch = usProcArch;
		m_bNeedProcArch = FALSE;
	};

	USHORT GetProcArch(){return m_usProcArch;};

	void SetNeedGuid(BOOL val) {m_bNeedGUID = val;};

	BOOL NeedGuid() {return m_bNeedGUID;};

	void GUID(const BYTE* pucGUID){memcpy(m_pucGUID, pucGUID, GUID_LENGTH);};

	void GetGUID(BYTE* pucGUID){memcpy(pucGUID, m_pucGUID, GUID_LENGTH);	};

/*	unsigned int Get43Option(unsigned char *pucOptions, unsigned int uiMaxLen );

	void Display (FILE* fp);
*/
};

#endif


