
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

typedef unsigned int DWORD;

#define MTFTPD_CONF_FILE "/etc/mtftpd.conf"

#define TRUE	1
#define FALSE	0

#define READ_SIZE	1024

static int ReadConfString (char *p_lpPath, char *p_lpField, char* p_pstrData);
static int ReadStringValue(FILE *fp, char* p_pstrData);
static int ReadConfDword (char *p_lpPath, char *p_lpField, DWORD *p_pdwVal);
static int ReadDwordValue(FILE *fp, DWORD* p_pdwVal);
static int FindConfEntry(FILE *fp, char *p_lpPath, char *p_lpField);
static int ReadNextValueLine(FILE* fp, char* buf);
static FILE* OpenMtftpdConfFile();

int 
IsMulticastEnabled()
{
	DWORD dwEnable;
	if (ReadConfDword(NULL, "IsMulticastEnabled", &dwEnable))
	{
		return dwEnable;
	}
	
	return 0;
}

unsigned short 
GetClientPort()
{

	DWORD dwValue;
	
	if (ReadConfDword(NULL, "ClientPort", &dwValue))
		return ((unsigned short)dwValue);
	
	return 0;
}

unsigned short 
GetServerPort()
{


	DWORD dwValue;
	
	if (ReadConfDword(NULL, "ServerPort", &dwValue))
		return ((unsigned short)dwValue);
	
	return 0;
}

unsigned char 
GetOpenTimeout()
{

	DWORD dwValue;
	
	if (ReadConfDword(NULL, "OpenTimeout", &dwValue))
		return ((unsigned char)dwValue);
	
	return 0;
}

unsigned char 
GetReopenDelay()
{

	DWORD dwValue;
	
	if (ReadConfDword(NULL, "ReopenDelay", &dwValue))
		return ((unsigned char)dwValue);
	
	return 0;
}

unsigned int 
GetMulticastAddress(unsigned int* pIp, char *pFilename)
{
	FILE* fp;
	char buf[READ_SIZE], temp[READ_SIZE], buf1[READ_SIZE];
	int ret;	
	fp = OpenMtftpdConfFile();
	if (fp == NULL)
		return FALSE;

	if (!FindConfEntry(fp, NULL, "Multicast_ip_addresses"))
	{
		fclose(fp);
		return FALSE;
	}
	while (1)
	{
		ret = ReadNextValueLine(fp, buf);
		if (ret == FALSE)
			break;
		if (strncmp(pFilename, buf, strlen(pFilename)) == 0)
		{
			sscanf(buf, "%s %s", buf1, temp);
			*pIp = inet_addr(temp);
			fclose(fp);
			return TRUE;
		}
	}
	fclose(fp);
	return FALSE;
}

char GetMulticastTTL()
{

	DWORD dwValue;
	
	if (ReadConfDword(NULL, "IP_Multicast_TTL", &dwValue))
		return ((char)dwValue);
	
	return 0;

}

static int
ReadConfString (char *p_lpPath, char *p_lpField, char* p_pstrData)
{
	FILE* fp;
	int ret;
	fp = OpenMtftpdConfFile();
	if (fp == NULL)
		return FALSE;

	if (!FindConfEntry(fp, p_lpPath, p_lpField))
	{
		fclose(fp);
		return FALSE;
	}
	ret = ReadStringValue(fp, p_pstrData);
	fclose(fp);
	return (ret)?TRUE:FALSE;
}

static int
ReadStringValue(FILE *fp, char* p_pstrData)
{
	p_pstrData[0] = 0; // initialize to NULL

	if (!ReadNextValueLine(fp, p_pstrData))
		return FALSE;

	return TRUE;
}

static int
ReadConfDword (char *p_lpPath, char *p_lpField, DWORD *p_pdwVal)
{
	FILE* fp;
	int ret;
	fp = OpenMtftpdConfFile();
	if (fp == NULL)
		return FALSE;

	if (!FindConfEntry(fp, p_lpPath, p_lpField))
	{
		fclose(fp);
		return FALSE;
	}
	ret = ReadDwordValue(fp, p_pdwVal);
	fclose(fp);
	return (ret)?TRUE:FALSE;

}

static int 
ReadDwordValue(FILE *fp, DWORD* p_pdwVal)
{
	char buf[READ_SIZE];

	buf[0] = 0; // initialize to NULL

	if (!ReadNextValueLine(fp, buf))
		return FALSE;

	*p_pdwVal = atoi(buf);
	return TRUE;
}


static int
FindConfEntry(FILE *fp, char *p_lpPath, char *p_lpField)
{
	char buf[READ_SIZE];
	char strToFind [READ_SIZE];
	int nLen;
	strcpy(strToFind, "[");
	if (p_lpPath != NULL)
	{
		strcat(strToFind, p_lpPath);
		strcat(strToFind, "/");
	}

	strcat(strToFind, p_lpField);

	nLen = strlen(strToFind);
	while(1)
	{
		fgets(buf, sizeof(buf), fp);
		if (feof(fp))
			break;
		if (strncasecmp(buf, strToFind,  nLen) == 0)
		{
			return TRUE;
		}

	}
	return FALSE;
}

static int
ReadNextValueLine(FILE* fp, char* buf)
{
	int nRead;

	buf[0] = 0; // initialize to NULL

	while(1)
	{
		fgets(buf, READ_SIZE, fp);
		if (feof(fp))
			break;
		nRead = strlen(buf);
		nRead--; // Account for the newline char which is part of the read
		if (nRead <= 0) // skip blank lines
			continue; 

		buf[nRead] = 0; // Remove the \n from the buffer

		if (buf[0] == '[')
		{
			// another registry entry has started. so bail out
			break;
		}
		if (buf[0] == '#')
		{
			// it is a comment line, so continue
			continue;
		}
		else
		{
			return TRUE;
		}
	}
	return FALSE;
}


static FILE*
OpenMtftpdConfFile()
{
	return fopen(MTFTPD_CONF_FILE, "r");
}
