
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include "pxemisc.h"
#include "dhcp.h"
#include "pxe.h"
#include "cdhcp.h"
#include "cproxreg.h"
#include "optsdll.h"

typedef struct _GUID {    
	unsigned long Data1;    
	unsigned short Data2;
    unsigned short Data3;    
	unsigned char Data4[8];
} GUID;

typedef GUID UUID;

USHORT GetGuid(BYTE* pucGUIDOpts, BYTE* ucMac,  int len);
void ParseMenu(CString p_strMenu, USHORT *p_usType, BYTE *p_ucLen,
						CString *p_strMenuItem);
BYTE htob(BYTE i);


extern "C" DWORD Options(PXE_OPTIONS_DATA* p_data)
{
	BYTE ucOptBuf[256];
	BYTE ucMacBuf[50];
	int nLen = 0;
	CProxyRegistry ProxyReg;
	int nStartPktSize = 0;
	
	static unsigned char NullGuid00[16] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
									0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	static unsigned char NullGuidFF[16] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
									0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

	// Convert byte arrays to CDhcp objects.
	CDhcp ClientPkt(p_data->ucClientPkt, p_data->usClientSize);
	CDhcp ReplyPkt(p_data->ucReplyPkt, p_data->usReplySize);

	nStartPktSize = p_data->usReplySize;

	
#if 0
	// UUID/GUID
	// If the client does not have a GUID create one and return it.
	nLen = ClientPkt.GetOption(DHCP_PLATFORMID, ucOptBuf, sizeof(ucOptBuf) );
	if (nLen && ucOptBuf[0] == DHCP_PLATFORMID_GUID)
	{	// There is a GUID.  Now test that it is a valid GUID.
		if ( (0 == memcmp(&ucOptBuf[1], &NullGuid00, 16))
			|| (0 == memcmp(&ucOptBuf[1], &NullGuidFF, 16)) )
		{
			// Not a valid GUID, create a new GUID.
			nLen = ClientPkt.HardwareAddress(ucMacBuf, sizeof(ucMacBuf));
			nLen =  GetGuid(ucOptBuf, ucMacBuf, sizeof(ucOptBuf) );
		} 
	}else	// There is no GUID, create a new GUID
	{
			nLen = ClientPkt.HardwareAddress(ucMacBuf, sizeof(ucMacBuf));
			nLen =  GetGuid(ucOptBuf, ucMacBuf, sizeof(ucOptBuf) );
	}
	// Put the GUID in the reply packet.
	ReplyPkt.AddOption(DHCP_PLATFORMID, ucOptBuf, nLen);
#endif

	{// MENU PROMPT
		CString strPrompt;
		CString strtmp1;
		CString strtmp2;
		int idx;
		BYTE ucTimeOut = 0;
		ProxyReg.GetPrompt(&strPrompt);
		// Parse out Timeout and convert to BYTE.
		idx = strPrompt.find(',');
		strtmp1 = strPrompt.substr(0, idx);
		strtmp2 = strPrompt.substr(idx+1, strPrompt.length());
		ucTimeOut = (BYTE) atoi(strtmp1);
		int iLen = strtmp2.length();

		// We are purposely not counting the NULL in the string so
		// the option will not include the NULL.

		// Convert CString to BYTE array.
		ucOptBuf[0] = ucTimeOut;
		strcpy((char *)&ucOptBuf[1], strtmp2);
		ReplyPkt.Add43Option(PXE_BOOT_PROMPT, ucOptBuf, iLen+1);
	}

	{// MENU ITEMS
		USHORT usArch = 0;
		CString strArch;
		CString strNicPath;
		BYTE pucNicType[50];

		CStringArray saMenu;
		int iNumItems;
		USHORT usType;
		BYTE ucLen;
		CString strMenuItem;
		int nOptLen = 0;
		USHORT usTypeNO;
		WORD wOptionsIndex = 0;

		// Extract Architecture type from client and convert to a registry path.
		ucLen = ClientPkt.GetOption(DHCP_SYSARCH, (BYTE*)&usArch, sizeof(usArch) );
		ProxyReg.GetArchPath(usArch, &strArch);

		// Extract NIC interface type from client and convert to a registry path.
		ucLen = ClientPkt.GetOption(DHCP_NICIF, pucNicType, sizeof(pucNicType) );
		ProxyReg.GetNicPath(pucNicType, strArch, &strNicPath);

		// Read Menu relative to the Arch path and Nic Path
		ProxyReg.GetMenu(strArch, strNicPath, &saMenu);
		iNumItems = saMenu.GetSize();

		if (iNumItems != 0)
		{
			for (int i=0; i < iNumItems; i++)
			{
				// Add each menu option to the packet
				ParseMenu(saMenu[i], &usType, &ucLen, &strMenuItem);
				usTypeNO = htons(usType);
				memcpy(&ucOptBuf[wOptionsIndex], &usTypeNO, 2);
				wOptionsIndex += 2;
				memcpy(&ucOptBuf[wOptionsIndex], &ucLen, 1);
				wOptionsIndex += 1;
				strcpy((char *)&ucOptBuf[wOptionsIndex], strMenuItem);
				wOptionsIndex += strMenuItem.length();
				nOptLen += strMenuItem.length() + 3;
			}
			ReplyPkt.Add43Option(PXE_BOOT_MENU, ucOptBuf, nOptLen );
		}
	}
	{ 
		// Convert the DHCP class object to a BYTE array for returning.
		int nPacketSize = ReplyPkt.PacketLength() + 100;
		BYTE* DhcpPacket = new (BYTE [nPacketSize]);
		memset(DhcpPacket, 0, nPacketSize);
		ReplyPkt.GetPacket(DhcpPacket, nPacketSize);
		if (nPacketSize <= p_data->usReplyMaxSize)
		{
			memcpy(p_data->ucReplyPkt, DhcpPacket, nPacketSize);
			p_data->usReplySize = nPacketSize;
		} else
		{	// Not all of the packet will fit so copy what will fit.
			memcpy(p_data->ucReplyPkt, DhcpPacket, p_data->usReplyMaxSize);
			p_data->usReplySize = p_data->usReplyMaxSize;
		}

		delete[] DhcpPacket;
	}
	return (p_data->usReplySize - nStartPktSize);
}

#if 0

USHORT 
GetGuid(BYTE* pucGUIDOpts, BYTE* ucMac,  int len)
{
	UUID uuid;
	BYTE *strUuid;

	char szUUID[50];
	BYTE ucUUID[50];

	if (len < 19)
		return 0;

	if (UuidCreate(&uuid) == RPC_S_OK)
	{
		UuidToString(&uuid, &strUuid);
	}
	_tcscpy(szUUID, (char *)strUuid);
	RpcStringFree(&strUuid);

	_tcsupr(szUUID);

	int i = 0;
	int j = 0;
	while ( szUUID[i] && (szUUID[i] != '\n'))
	{
		if (szUUID[i] != '-')
		{
			ucUUID[j] = (htob(szUUID[i])*0x10) + htob(szUUID[i+1]);
			j++;
			i++;
		}
		i++;
		if (i>35)
			break;

	}

	pucGUIDOpts[0] = DHCP_PLATFORMID_GUID;
	memcpy (&pucGUIDOpts[1], ucUUID, 10);
	memcpy (&pucGUIDOpts[11], ucMac, 6);


	return 17;
}
#endif

BYTE
htob(BYTE i)
{
	unsigned char hexes[17] = {0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F};

	if ((i > 0x2F) && (i < 0x3f))
		return (i-0x30);
	else
		return (hexes[i-0x41]);

}

void
ParseMenu(CString p_strMenu, USHORT *p_usType, BYTE *p_ucLen,
						CString *p_strMenuItem)
{
	CString strSrc = p_strMenu;
	CString strtmp1;
	CString strtmp2;
	int iSepPos;

	iSepPos = strSrc.find(',');
	strtmp1 = strSrc.substr(0, iSepPos);
	strtmp2 = strSrc.substr(iSepPos+1, strSrc.length());
	*p_usType = (USHORT) atoi(strtmp1);
	*p_ucLen = strtmp2.length();
	*p_strMenuItem = strtmp2;

	return;
}
