
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include "pxemisc.h"

#include "cdhcp.h"
#include "dhcppkt.h"
#define MAXDISCLEN 1500  //Used by pktdata
#include "pktdata.h"
#include "pxe.h"
#include "parsedll.h"
#include "optsdll.h"
#include "cpxedebug.h"
#include "cipaddr.h"
#include <dlfcn.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#define BOOTP_DHCPVEND  312     // DHCP standard vendor field size
#define MIN_DHCP_SIZE   548
#define DHCP_FIXEDPART_SIZE (MIN_DHCP_SIZE-BOOTP_DHCPVEND)

#define VALID	0
#define MISSING	1
#define INVALID 2

FILE *fpTest1 = NULL;
FILE *fpTest2 = NULL;

void TestPrintGuid(CStringArray& aryBuf, const char *str, BYTE* GUID);
void TestPrintNic(CStringArray& aryBuf, const char *str, BYTE* NI);
void TestPrint(CStringArray& aryBuf, const char *str);
void TestPrint(CStringArray& aryBuf, const char* format, const char *str);
void TestPrint(FILE *fp, const char* format, const char *str);
void DebugPrint(const char *str);
void OutputTest(FILE *fp, CStringArray& aryBuf);
void CleanUp();

void sHexDump(CStringArray& p_OutBuf, const unsigned char* buf, unsigned int uilen)
{
	unsigned char hexchar[]="0123456789ABCDEF";	// I18N
	unsigned char lne[82];
	unsigned int hex;
	unsigned int txt;

	unsigned int index = 0;
	do
	{
		memset(lne, ' ', 80);
		lne[79] = '\n';
		lne[80] = '\0';
		hex=11;
		txt=60;

		sprintf((char *)lne, "     %04x  ", index);	// I18N
		for (int i=0; ((i<16) & (index < uilen)); i++)
		{
			lne[hex++]=hexchar[(buf[index] >> 4)];
			lne[hex++]=hexchar[(buf[index] & 0xF)];
			hex++;

			lne[txt++]=(buf[index] < 31 || buf[index] == 127)?'.':buf[index];
			index++;
		}

		p_OutBuf.Add((char *)lne);

	} while (index < uilen);

}

CPxeDebug Pxedebug;
extern "C" DWORD
Initialize()
{
	CProxyRegistry ProxyReg;
	CStringArray csNames;

	Pxedebug.init(PXE_TESTER_DEBUG);

	// In PxeclientTester, we would try to create a directory under the
	// TESTLOG share directory. We connect to this by \\servername\testlog UNC
	// connection. However, because of the way NT services function, this 
	// would fail with Access Denied as by default, the NT services operate
	// with NULL credentials. So, we need to add the testlog share name as a 
	// permissible one under LANMANServer key. (I found this from
	// an article on MSDN knowledge base

#define LANMAN_KEY_NAME	"SYSTEM\\CurrentControlSet\\Services\\LanmanServer\\Parameters"

#if 0
	ProxyReg.ReadRegMultiString(LANMAN_KEY_NAME, "NullSessionShares", &csNames);
	BOOL bFound = FALSE;

	// See whether the TestLog share name is already in the registry key. Otherwise
	// add to the list
	for (int i=0; i < csNames.GetSize(); i++)
	{
		if ((csNames.GetAt(i)).CompareNoCase("TestLog") == 0)
		{
			bFound = TRUE;
			break;
		}
	}
	if (!bFound)
	{
		// add it to the registry
		csNames.Add("TestLog");
		ProxyReg.WriteRegMultiString(LANMAN_KEY_NAME, "NullSessionShares", csNames);
	}
#endif
	return 1;

}

extern "C" DWORD
PXEClientTester(PXE_PARSER_DATA *lpParam)
{
	CString strTestPath;
	CProxyRegistry ProxyReg;
	//Ensure testing is activated.
	if (! (ProxyReg.TestOn()))
	{
		return PXE_PACKET_NOT_SENT;
	}
	// Ensure a TestPath is defined.
	if (!ProxyReg.TestPath(strTestPath))
	{
		return PXE_PACKET_NOT_SENT;
	}
	PXE_PARSER_DATA pd;
	// Make a local copy of the packet.
	memcpy(&pd, lpParam, sizeof(pd));
	// Convert our copy of the packet to a CDhcp object.
	CDhcp ClientPkt((BYTE*)pd.ucPacket, pd.nPacketLength);

//-----------------------------------------------------------

	time_t tt;
	time(&tt);
	CString strTime = ctime(&tt);

	BYTE pucMACBuf[50];
	int nMACLen = 0;
	CString strMac;
	CString strMacFileName;

	BYTE ucMsgType;
	int nMsgLen = 0;
	BYTE ucMsgValid = MISSING;

	BYTE pucClassBuf[50];
	unsigned int usnClassLen = 0;
	BYTE ucClass = MISSING;

	USHORT usArch = 0xFF;
	int nArchLen = 0;
	BYTE ucSysArch = MISSING;

	BYTE pucNicInterface[50];
	int nNicLen = 0;
	BYTE ucNicIf = MISSING;

	BYTE pucGuidBuf[50];
	int nGuidLen = 0;
	BYTE ucPlatformId = MISSING;

	static unsigned char NullGuid00[16] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
									0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	static unsigned char NullGuidFF[16] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
									0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

	BYTE ucPXEBoot[50];
	int nPXEBootLen = 0;

	USHORT GetGuid(BYTE* pucGUIDOpts, BYTE* ucMac,  int len);

	char szTmpBuf[512];

	CStringArray TestBuffer1;
	CStringArray TestBuffer2;


	// Set the number of elements to 100
	TestBuffer1.SetSize(100);
	TestBuffer2.SetSize(100);

//=======================================================

	// Get Message Type
	nMsgLen = ClientPkt.GetOption(DHCP_MSGTYPE, &ucMsgType, sizeof(ucMsgType) );

	// Only process BINL and Proxy packets
	if ((pd.usReceivePort == 4011 )
		|| (pd.usReceivePort == 67 && ucMsgType == DHCP_DISCOVER)
		)
	{
		// This packet is eligible for packet analysis
	}
	else
	{
		// Shouldn't process this packet
		Pxedebug.write("PxeClientTester: Not processing packet with DHCP msg-type %d received on port %d\n",
			ucMsgType, pd.usReceivePort);
		CleanUp();
		return PXE_PACKET_NOT_SENT;
	}

	// Don't process packets that are boot server discovery packets.
	nPXEBootLen =  ClientPkt.Get43Option(PXE_BOOT_ITEM, ucPXEBoot, sizeof(ucPXEBoot) );
	if (nPXEBootLen)
	{
		CleanUp();
		return PXE_PACKET_NOT_SENT;
	}

	DebugPrint ( "ClientTestParser: packet passed all initial tests.\n");

	// Get Client MAC Address
	nMACLen = ClientPkt.HardwareAddress(pucMACBuf, sizeof(pucMACBuf));
	if (nMACLen)
	{
		char tmp1[1024];
		sprintf(tmp1, "%02X%02X%02X%02X%02X%02X", pucMACBuf[0], pucMACBuf[1], pucMACBuf[2], pucMACBuf[3], pucMACBuf[4], pucMACBuf[5]);
		strMac = tmp1;
		sprintf(tmp1, "%02X%02X%02X%02X",  pucMACBuf[2], pucMACBuf[3], pucMACBuf[4], pucMACBuf[5]);
		strMacFileName = tmp1;
	} else
	{
		strMac = "NO MAC ADDRESS";
	}

	//===========================================================================================
//------Start Print Reports---------------------------------
// Print  Report Header Lines

	TestPrint(TestBuffer1, "\n======================================================================\n");
	sprintf(szTmpBuf, "%s : Start of Packet Analysis For Client:%s\n", strTime.data(), strMac.data());
	TestPrint(TestBuffer1, szTmpBuf);
	TestPrint(TestBuffer1, "======================================================================\n");

	TestPrint(TestBuffer2, "\n======================================================================\n");
	sprintf(szTmpBuf, "%s : Results of Packet Analysis For Client:%s\n", strTime.data(), strMac.data());
	TestPrint(TestBuffer2, szTmpBuf);
	TestPrint(TestBuffer2, "======================================================================\n");

	TestPrint(TestBuffer1, "----------------------------------------------------------------------\n");
	sprintf(szTmpBuf, "Fixed portion of DHCP Packet sent by client:%s\n", strMac.data());
	TestPrint(TestBuffer1, szTmpBuf);
	TestPrint(TestBuffer1, "----------------------------------------------------------------------\n");
	sHexDump(TestBuffer1, (BYTE*)pd.ucPacket, DHCP_FIXEDPART_SIZE );

	TestPrint(TestBuffer1, "\n----------------------------------------------------------------------\n");
	sprintf(szTmpBuf, "DHCP Options sent by client:%s\n", strMac.data());
	TestPrint(TestBuffer1, szTmpBuf);
	TestPrint(TestBuffer1, "----------------------------------------------------------------------\n");
	sHexDump(TestBuffer1, &(pd.ucPacket[DHCP_FIXEDPART_SIZE]), pd.nPacketLength-DHCP_FIXEDPART_SIZE);

	TestPrint(TestBuffer1, "\n----------------------------------------------------------------------\n");
	TestPrint(TestBuffer1, "\t\tPACKET ANALYSIS\n");
	TestPrint(TestBuffer1, "----------------------------------------------------------------------\n");

	TestPrint(TestBuffer1,"\n");
//===========================================================================================
	// Validate Message Type
	if (ucMsgType == DHCP_DISCOVER)
	{
		TestPrint(TestBuffer1, "DHCP Message Type(53,0X35): DHCP_DISCOVER\n");
		ucMsgValid = VALID;
	} else if (ucMsgType == DHCP_REQUEST)
	{
		TestPrint(TestBuffer1, "DHCP Message Type(53,0X35): DHCP_REQUEST\n");
		ucMsgValid = VALID;
	} else if (ucMsgType == DHCP_INFORM)
	{
		TestPrint(TestBuffer1, "DHCP Message Type(53,0X35): DHCP_INFORM\n");
		ucMsgValid = VALID;
	} else 
	{
		TestPrint(TestBuffer1, "DHCP Message Type(53,0X35): Unknown\n");
		ucMsgValid = INVALID;
	}

	//Get Client Class
	usnClassLen = ClientPkt.GetOption(DHCP_CLASS, pucClassBuf, sizeof(pucClassBuf) );
	if (usnClassLen == 0)
	{// No tag
		ucClass = MISSING;
	} else 	if (usnClassLen >= strlen(PXE_CLS_CLIENT))
	{
		if ( strncmp((char*)pucClassBuf, PXE_CLS_CLIENT, strlen(PXE_CLS_CLIENT) ) == 0)
		{
			TestPrint(TestBuffer1, "DHCP Class ID(60,0X3C): PXE Client\n");
			ucClass = VALID;
		}
	} else
	{
		TestPrint(TestBuffer1, "DHCP Class ID(60,0X3C): NON-PXE Client\n");
		ucClass = INVALID;
	}

	CString csExtTag;
	if (ClientPkt.GetExtendedPxeClientTag(&csExtTag))
	{
		TestPrint(TestBuffer1, "DHCP PXEClient Extended Tag present: ");
		TestPrint(TestBuffer1, "%s\n", (char*)((const char*)csExtTag));
	}

	// Get Client Architecture
	nArchLen = ClientPkt.GetOption(DHCP_SYSARCH, (BYTE*)&usArch, sizeof(usArch) );
	if (nArchLen == 0)
	{
		ucSysArch = MISSING;

	//} else 	if (nArchLen == 2 && (usArch >= 0 && usArch <=4 ) )
	} else 	if (nArchLen == 2 &&  usArch <=4 ) 
	{	// Convert from network order.
		usArch =  ntohs(*(USHORT*) &usArch );
		switch (usArch)
		{
		case 0: TestPrint(TestBuffer1, "DHCP SYSTEM ARCH(93,0X5D): X86PC\n"); break;
		case 1: TestPrint(TestBuffer1, "DHCP SYSTEM ARCH(93,0X5D): NEC/PC98\n"); break;
		case 2: TestPrint(TestBuffer1, "DHCP SYSTEM ARCH(93,0X5D): IA64 PC\n"); break;
		case 3: TestPrint(TestBuffer1, "DHCP SYSTEM ARCH(93,0X5D): DEC Alpha\n"); break;
		case 4: TestPrint(TestBuffer1, "DHCP SYSTEM ARCH(93,0X5D): ArcX86\n"); break;
		}
		ucSysArch = VALID;

	} else
	{
		TestPrint(TestBuffer1, "DHCP SYSTEM ARCH(93,0X5D): UNKNOWN\n");
		ucSysArch = INVALID;
	}

	CString csOpt;
	BOOL bOk;
	if (ClientPkt.GetOldStyleOpt93(&csOpt, &bOk))
	{
		if (!bOk)
		{
			TestPrint(TestBuffer1, "DHCP SYSTEM ARCH(93,0X5D) Option and the info in PXEClient extended tag do not match\n");
		}
		
	}

	// Get NIC Interface
	nNicLen = ClientPkt.GetOption(DHCP_NICIF, pucNicInterface, sizeof(pucNicInterface) );
	if (nNicLen == 0)
	{
		ucNicIf = MISSING;
	} else 	if (pucNicInterface[0] == 1 && pucNicInterface[1] == 2 && pucNicInterface[2] == 1)
	{
		TestPrintNic(TestBuffer1, "DHCP NICIF (94,0X5E): ", pucNicInterface);
		ucNicIf = VALID;
	} else
	{
		TestPrintNic(TestBuffer1, "INVALID DHCP NICIF (94,0X5E): ", pucNicInterface);
		ucNicIf = INVALID;
	}

	if (ClientPkt.GetOldStyleOpt94(&csOpt, &bOk))
	{
		if (!bOk)
		{
			TestPrint(TestBuffer1, "DHCP NICIF (94,0X5E) Option and the info in PXEClient extended tag do not match\n");
		}
		
	}
	// Get GUID
	nGuidLen = ClientPkt.GetOption(DHCP_PLATFORMID, pucGuidBuf, sizeof(pucGuidBuf) );
	// Test GUID
	if (nGuidLen == 0)
	{
			ucPlatformId = MISSING;
	} else 	if (nGuidLen == 17)
	{	// There is a GUID.  Now test that it is a valid GUID.
		if ( (0 == memcmp(pucGuidBuf, &NullGuid00, 16))
			|| (0 == memcmp(pucGuidBuf, &NullGuidFF, 16)) )
		{
			// Not a valid GUID
			TestPrintGuid(TestBuffer1, "INVALID DHCP PLATFORM ID (97,0X61): ", pucGuidBuf);
			ucPlatformId = INVALID;

		} else
		{
			TestPrintGuid(TestBuffer1, "DHCP PLATFORM ID (97,0X61): ", pucGuidBuf);
			ucPlatformId = VALID;
		}
	}

//------End Get Data Items---------------------------------
	if (ucMsgValid)
	{
		TestPrint(TestBuffer1, "\t%s DHCP Message Type (53,0x35)\n", 
			((ucMsgValid == MISSING) ? "MISSING" : "INVALID"));
		TestPrint(TestBuffer2, "\t DHCP Message Type (53,0x35)\n", 
			(ucMsgValid == MISSING) ? "MISSING" : "INVALID");
	}
	if (ucClass)
	{
		TestPrint(TestBuffer1, "\t%s Vendor Class Identifier (60,0x3C)\n",
			(ucClass == MISSING) ? "MISSING" : "INVALID");
		TestPrint(TestBuffer2, "\tInvalid or missing Vendor Class Identifier (60,0x3C)\n",
			(ucClass == MISSING) ? "MISSING" : "INVALID");
	}
	if (ucSysArch)
	{
		TestPrint(TestBuffer1, "\t%s Client System Architecture (93,0x5D)\n",
			(ucSysArch == MISSING) ? "MISSING" : "INVALID");
		TestPrint(TestBuffer2, "\t%s Client System Architecture (93,0x5D)\n",
			(ucSysArch == MISSING) ? "MISSING" : "INVALID");
	}
	if (ucNicIf)
	{
		TestPrint(TestBuffer1, "\t%s Client Network Interface Identifier (94,0x5E)\n",
			(ucNicIf == MISSING) ? "MISSING" : "INVALID");
		TestPrint(TestBuffer2, "\t%s Client Network Interface Identifier (94,0x5E)\n",
			(ucNicIf == MISSING) ? "MISSING" : "INVALID");
	}
	if (ucPlatformId)
	{
		TestPrint(TestBuffer1, "\t%s Client Machine Identifier(GUID) (97,0x61)\n",
			(ucPlatformId == MISSING) ? "MISSING" : "INVALID");
		TestPrint(TestBuffer2, "\t%s Client Machine Identifier(GUID) (97,0x61)\n",
			(ucPlatformId == MISSING) ? "MISSING" : "INVALID");
	}

	if (ucMsgValid || ucClass || ucSysArch || ucNicIf || ucPlatformId)
	{
		TestPrint(TestBuffer1, "\nFAILED: Packet does not comply with PXE protocol "\
								"as specified in \n\tPXE Specification Version "\
								"2.0 due to the errors listed above.\n");
		TestPrint(TestBuffer2,  "\nFAILED: Packet does not comply with PXE protocol "\
								"as specified in \n\tPXE Specification  Version "\
								"2.0 due to the errors listed above.\n");
	} else
	{
		TestPrint(TestBuffer1, "\nSUCCESS: Packet complies with PXE protocol as "\
								"specified in \n\tPXE Specification  Version 2.0.\n");
		TestPrint(TestBuffer2, "\nSUCCESS: Packet complies with PXE protocol as "\
								"specified in \n\tPXE Specification  Version 2.0.\n");
	}
	TestPrint(TestBuffer1, "\n----------------------------------------------------------------------\n");
	TestPrint(TestBuffer1, "\t\tEND OF ANALYSIS\n");
	TestPrint(TestBuffer1, "----------------------------------------------------------------------\n");
	TestPrint(TestBuffer1, "\n*******************************************************************\n\n");
	TestPrint(TestBuffer2, "\n----------------------------------------------------------------------\n");
	TestPrint(TestBuffer2, "\t\tEND OF ANALYSIS\n");
	TestPrint(TestBuffer2, "----------------------------------------------------------------------\n");
	TestPrint(TestBuffer2, "\n*******************************************************************\n\n");

	// Attempt to open the test output files
	{
		Pxedebug.logError("Writing to Test files\n");
		strTestPath = strTestPath + "/" + strMacFileName;
		int result=0;
		DWORD reserr=0;
		result = mkdir(strTestPath.data(), 755);
		if (result!=0)
		{
			reserr=errno;
			if (EEXIST != reserr)
			{
				Pxedebug.logError("PxeClientTester: CreateDirectory of %s failed with error 0x%x\n", 
					strTestPath.data(), reserr);
			}

		}

		// ATTEMPT TO OPEN FILES EVEN IF CreateDirectory Failed
		CString csPath1, csPath2;
		csPath1 = strTestPath + "/DHCPPkt.txt";
		fpTest1 = fopen(csPath1, "a");
		if (fpTest1 == NULL)
		{
			Pxedebug.logError("fopen failed\n");
		}
		csPath2 = strTestPath + "/TestSum.txt";
		fpTest2 = fopen(csPath2, "a");
	}
	
	// If the test files are open print the output and close them.
	if (fpTest1)
	{
		OutputTest(fpTest1, TestBuffer1);
		fclose(fpTest1);
		fpTest1 = NULL;
	}
	if (fpTest2)
	{
		OutputTest(fpTest2, TestBuffer2);
		fclose(fpTest2);
		fpTest2 = NULL;
	}
	CleanUp();
	return PXE_PACKET_NOT_SENT;
}


unsigned char htob(unsigned char i)
{
	unsigned char hexes[17] = {0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F};

	if ((i > 0x2F) && (i < 0x3f))
		return (i-0x30);
	else
		return (hexes[i-0x41]);

}

void TestPrint(CStringArray& aryBuf, char *str)
{
	char * tmp = new char[strlen(str)+1];
	sprintf(tmp, "%s", str);
	aryBuf.Add(tmp);
	delete tmp;
}

void TestPrint(CStringArray& aryBuf, char* format, char *str)
{
	char * tmp = new char[255];
	sprintf(tmp, format, str);
	aryBuf.Add(tmp);
	delete tmp;
}

void TestPrintGuid(CStringArray& aryBuf, char *str, BYTE* GUID)
{
	char tmp[1024];
	
	CString GuidTxt;
	sprintf(tmp, "%s%02X%02X02X%02X-%02X%02X-%02X%02X-%02X%02x-%02X%02X%02X%02X%02X%02X\n",
		str?str:"", GUID[1], GUID[2], GUID[3], GUID[4], GUID[5], GUID[6], GUID[7], GUID[8], GUID[9],
						GUID[10], GUID[11], GUID[12], GUID[13], GUID[14], GUID[15], GUID[16]);

	GuidTxt = tmp;
	aryBuf.Add(GuidTxt);
}
#define UNDI 1
#define PCI 2
#define PNP 3

void TestPrintNic(CStringArray& aryBuf, char *str, BYTE* NI)
{
	CString NicType;
	char tmpb[1024];

	switch (NI[0])
	{
	case  UNDI:
			sprintf(tmpb, "%sUNDI-%02X-%02X\n",
						str,
						NI[1],
						NI[2]);
			NicType = tmpb;
			break;
#if 0
	case  PCI:
			NicType.Format(_T("%sPCI-%02X%02X-%02X%02X-%02X%02X%02X-%02X-%02X%02x-%02X%02x\n"),
						str, NI[1], NI[2], NI[3], NI[4], NI[5], NI[6], NI[7], NI[8], NI[9],
						NI[10], NI[11], NI[12]);
						
			break;

	case  PNP:
			NicType.Format(_T("%s%c%c%c-%1X%1X%1X%1X-%02X%02X%02X\n"),	
						str,
						( ( (NI[1] & 0x7C) >>2 ) | 0x40),
						( ( ( (NI[1] & 0x03) << 3 ) | ((NI[2] &0xE0) >> 5)) | 0x40),
						( (NI[2] & 0x1F) | 0x40 ),
						( (NI[3] & 0xF0) >> 4 ),
						  (NI[3] & 0x0F),
						( (NI[4] & 0xF0) >> 4 ),
						  (NI[4] & 0x0F),
						NI[5], NI[6], NI[7]
					);
			break;
#endif
	default:
		sprintf(tmpb, "%sUNKNOWN\n", str);
		NicType = tmpb;
		break;
	}

	aryBuf.Add(NicType);

}

void OutputTest(FILE *fp, CStringArray& aryBuf)
{
	int nUpper = aryBuf.GetSize();
	for (int i = 0; i<nUpper; i++)
	{
		fprintf(fp, "%s", aryBuf[i].data());
	}
	fflush(NULL);
}

void TestPrint(FILE *fp, char* format, char *str)
{
	if (fp)
	{
		fprintf(fp, format, str);
		fflush(NULL);
	}
}

void TestPrint(FILE *fp, char *str)
{
	if (fp)
	{
		fprintf(fp, "%s", str);
		fflush(NULL);
	}
}

void TestPrintNic(FILE *fp, char *str, BYTE* NI)
{
	char tmpb[1024];
	CString NicType;
	if (fp)
	{
		switch (NI[0])
		{
		case  UNDI:
				sprintf(tmpb, "UNDI-%02X-%02X",
							NI[1],
							NI[2]);
				NicType = tmpb;
				break;
#if 0
		case  PCI:
				NicType.Format(_T("PCI-%02X%02X-%02X%02X-%02X%02X%02X-%02X-%02X%02x-%02X%02x"),
							NI[1], NI[2], NI[3], NI[4], NI[5], NI[6], NI[7], NI[8], NI[9],
							NI[10], NI[11], NI[12]);
							
				break;

		case  PNP:
				NicType.Format(_T("%c%c%c-%1X%1X%1X%1X-%02X%02X%02X"),	
							( ( (NI[1] & 0x7C) >>2 ) | 0x40),
							( ( ( (NI[1] & 0x03) << 3 ) | ((NI[2] &0xE0) >> 5)) | 0x40),
							( (NI[2] & 0x1F) | 0x40 ),
							( (NI[3] & 0xF0) >> 4 ),
							  (NI[3] & 0x0F),
							( (NI[4] & 0xF0) >> 4 ),
							  (NI[4] & 0x0F),
							NI[5], NI[6], NI[7]
						);
				break;
#endif
		default:
			NicType = "UNKNOWN";
			break;
		}

		fprintf(fp, "%s%s\n", str, NicType.data());
		fflush(NULL);
	}

}

void DebugPrint(char *str)
{
	Pxedebug.write(str);
}

void CleanUp()
{
	if (fpTest1)
		fclose(fpTest1);
	if (fpTest2)
		fclose(fpTest2);
	fpTest1 = fpTest2 = NULL;
}
