
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include "pxemisc.h"

#include "cdhcp.h"
#include "dhcppkt.h"
#define MAXDISCLEN 1500  //Used by pktdata
#include "pktdata.h"
#include "pxe.h"
#include "parsedll.h"
#include "optsdll.h"
#include "cpxedebug.h"
#include "cipaddr.h"
#include <dlfcn.h>

#define OPTIONS_FUNCTION "Options"

#define BOOTREPLY               ((BYTE)2)
#define IPLEN 4

// Helper Functions
BOOL GetHostIPAddress(BYTE *bHostIP, int size);
extern void ParseBootServers(CString p_strServer, USHORT *p_usType, BYTE *p_ucLen,
					  CString *pIPs);

extern "C" DWORD BstrapOptions(PXE_OPTIONS_DATA* p_data);

// MTFTPD related functions that are implemented as just C (not C++)
// functions so that they can be used by the 'C' version of mtftpd.

extern "C" int IsMulticastEnabled();
extern "C" USHORT GetClientPort();
extern "C" USHORT GetServerPort();
extern "C" BYTE GetOpenTimeout();
extern "C" BYTE GetReopenDelay();
extern "C" DWORD GetMulticastAddress(DWORD* pdwMtftpIp, char *pFilename);
CPxeDebug parserDbg;

extern "C" DWORD
Initialize()
{
	static int bInitialized = FALSE;
	if (bInitialized)
		return TRUE;
	bInitialized = TRUE;
	parserDbg.init(PXE_PARSER_DEBUG);
	return 1;

}

extern "C" DWORD
PxeParser(PXE_PARSER_DATA *lpParam)
{
	Initialize();

	PXE_PARSER_DATA pd;
	// Make a local copy of the packet.

	memcpy(&pd, lpParam, sizeof(pd));
	//SetEvent(pd.hEvent);
	// Point to our copy of the packet.
	CDhcp ClientPkt((BYTE*)pd.ucPacket, pd.nPacketLength);

	PXE_OPTIONS_DATA OptStruct;
	PXE_OPTIONS_FUNC OptFunc;

	BOOL bResult = FALSE;

	BYTE ucHostIP[5];
	BYTE ucOptBuf[256];
	BYTE ucMsgType;
	BYTE ucLen;
	UINT nLen = 0;
	CDhcp ReplyPkt;
	USHORT usMaxPacketSize = 0;
	int usPacketSize = 0;
	USHORT usArch = 0;
	BYTE pucNicInterface[50];
	USHORT usService = 0;
	USHORT usLayer = 0;
	CString strBootFile;
	BYTE* pucDhcpPacket = NULL;
	BOOL bMcastDiscoveryEnabled = TRUE;

	CProxyRegistry ProxyReg;
	//CTFTPDReg MTFTPDReg(NULL);
	//BOOL f_bBinlMode = ProxyReg.StartBinlFlag();
	BOOL f_bProxyRunning = ProxyReg.StartProxyFlag();
	BOOL f_bBootServerRunning = ProxyReg.StartBootServiceFlag();

	if (parserDbg.DebugOn())
	{
		parserDbg.write("PxeParser: startProxy=%d, startBootServer=%d\n", 
			f_bProxyRunning, f_bBootServerRunning);
	}
	// Flag to indicate whether we are sending a response.
	BOOL bSendResponse = FALSE;

	if (parserDbg.DebugOn())
	{
		parserDbg.write("PxeParser: Packet received on port %d.\n", pd.usReceivePort);
	}

	if ( ! (f_bProxyRunning || f_bBootServerRunning))
	{
		return PXE_PACKET_NOT_SENT;
	}

	// Verify this is our Packet
	// Verify the port we received it on.
	if (pd.usReceivePort != 67 && pd.usReceivePort != 4011)
	{
		return PXE_PACKET_NOT_SENT;
	}

	// Verify the DHCP class is "PXEClient"
	nLen = ClientPkt.GetOption(DHCP_CLASS, ucOptBuf, sizeof(ucOptBuf) );
	if (nLen != strlen(PXE_CLS_CLIENT))
	{
		return PXE_PACKET_NOT_SENT;
	}
	if ( strncmp((char*)ucOptBuf, PXE_CLS_CLIENT, strlen(PXE_CLS_CLIENT) ) != 0)
	{
		return PXE_PACKET_NOT_SENT;
	}

	// Verify message type.
	nLen = ClientPkt.GetOption(DHCP_MSGTYPE, &ucMsgType, sizeof(ucMsgType) );
	if (nLen != 1)
	{
		return PXE_PACKET_NOT_SENT;
	}
	if (ucMsgType != DHCP_DISCOVER
		&& ucMsgType != DHCP_REQUEST
		&& ucMsgType != DHCP_INFORM)
	{
		return PXE_PACKET_NOT_SENT;
	}

	if ( pd.usReceivePort == 67 && ucMsgType == DHCP_REQUEST && ClientPkt.ClientIP() == 0)
	{
		// Drop this packet as the client is not in a IP address BOUND state and is just
		// sending a DHCPREQUEST packet to a DHCP server to confirm its selected IP Address
		// We should not be responding to this packet.
		parserDbg.write(_T("PxeParser: Packet not processed as it is received on 67 with ciaddr==0\n"));
		return PXE_PACKET_NOT_SENT;
	}

	// Now, find out what kind of request we received. (proxyDHCP or Bootserver)
	BOOL bBootServerReq = FALSE;
	BOOL bProxyDHCPReq = FALSE;

	// for a Pre PXE 1.0 clients, there is no concept of boot servers and proxy. 
	// so, we need to provide them with a boot file name. Whenever, we receive a proxyDHCP 
	// request, we will assume that a file name needs to be provided too,
	BOOL bProvideBstrapFileName = FALSE;
	ucLen = ClientPkt.Get43Option(PXE_BOOT_ITEM, ucOptBuf, sizeof(ucOptBuf) );
	if (ucLen == 4)
	{
		// this means that the boot_item option is present. Hence this is a boot server
		// related request.
		bBootServerReq = TRUE;
		parserDbg.write("PxeParser: This is a PXE boot server request packet\n");
	}
	else
	{
		// Verify whether this is a proxy related requests for general options
		// In this case, we accept DHCP_DISCOVER packets or 
		// DHCP_REQUEST and DHCP_INFORM packets on 4011
		if ((ucMsgType == DHCP_DISCOVER) ||
			(pd.usReceivePort == 4011 && ucMsgType == DHCP_REQUEST) ||
			(pd.usReceivePort == 4011 && ucMsgType == DHCP_INFORM))
		{
			bProxyDHCPReq = TRUE;
			if (f_bProxyRunning)
			{
				bProvideBstrapFileName = TRUE;
			}
			parserDbg.write("PxeParser: This is a proxyDHCP request packet\n");
		}
		else
		{
			parserDbg.write("PxeParser: Neither a boot request nor a proxy request\n");
		}

	}

	// Initialize reply packet with the basics.
	ReplyPkt.Opcode(BOOTREPLY);
	ReplyPkt.Hardware(ClientPkt.Hardware());
	ReplyPkt.YourIP(ClientPkt.YourIP());
	ReplyPkt.HardwareLength(ClientPkt.HardwareLength());
	ReplyPkt.TransactionID(ClientPkt.TransactionID());
	ReplyPkt.Seconds(ClientPkt.Seconds());
	ReplyPkt.GatewayIP(ClientPkt.GatewayIP());
	// Preserve the flags that the client set
	ReplyPkt.Flags(ClientPkt.Flags());

	// Set MAC address in DHCP header.
	{
		BYTE nHwLen = 0;
		nHwLen = ClientPkt.HardwareAddress(ucOptBuf, sizeof(ucOptBuf));
		if (nHwLen)
		{
			ReplyPkt.SetHardwareAddress(ucOptBuf, nHwLen);
		}
	}

	if (ucMsgType == DHCP_DISCOVER)
	{
		ucOptBuf[0] = DHCP_OFFER;
	} else
	{
		ucOptBuf[0] = DHCP_ACK;
	}
	ReplyPkt.AddOption(DHCP_MSGTYPE, ucOptBuf, 1);

	ULONG ulServerIP = 0;
	ulServerIP = ProxyReg.GetDiscoverySrvrIP();
	if ( ulServerIP)
	{
		ReplyPkt.AddOption(DHCP_SRVID, (BYTE*)&ulServerIP, 4);
	} else
	{
		//DHCP_SERVID - address of this server.
		CString csIp;
		if (!ProxyReg.GetSystemIPAddress(&csIp))
		{
			parserDbg.logError(ERRLOG_GENERIC_ERROR, "PxeParser: Unable to get our own ip address\n");
			
		}
		else
		{
			CIPAddr cipaddr(csIp);
			DWORD cipdword = cipaddr.GetIP();
			memcpy(ucHostIP, (char*)&cipdword, 4);
			ReplyPkt.AddOption(DHCP_SRVID, ucHostIP, 4);
		}
	}
	if (ulServerIP == 0)
		ulServerIP = *(ULONG*)ucHostIP;


	//DHCP_CLASS
	strcpy((char*)ucOptBuf, PXE_CLS_CLIENT);
	ReplyPkt.AddOption(DHCP_CLASS, ucOptBuf, strlen(PXE_CLS_CLIENT));

	// PROXY/BINL
	// Do we return client configuration Options?
	if ( f_bProxyRunning && bProxyDHCPReq ) 
	{	// Insert configuration options.
		bSendResponse = TRUE;
		{// PXE DISCOVERY CONTROL
			BYTE ucFlags = 0;
			if (ProxyReg.GetBcastFlag())
			{
				ucFlags |= 0x01;
			}
			if (ProxyReg.GetMcastFlag())
			{
				ucFlags |= 0x02;
				bMcastDiscoveryEnabled = FALSE;
			}
			if (ProxyReg.GetServerListFlag())
			{
				ucFlags |= 0x04;
			}
			if (ProxyReg.GetBStrapFlag())
			{
				ucFlags |= 0x08;
			}
			ReplyPkt.Add43Option(PXE_DISCOVERY_CONTROL, (BYTE *)&ucFlags, 1);
		}

		if (bMcastDiscoveryEnabled)
		{// PXE MULTICAST DISCOVERY ADDRESS
			ULONG ulSrvrAddr = ProxyReg.GetDiscoveryAddr();
			ReplyPkt.Add43Option(PXE_MCAST_DISCOVERY_ADDR, (BYTE *)&ulSrvrAddr, 4);
		}

		{// LIST OF BOOT SERVERS
			int iNumItems;
			CStringArray saServers;
			CString IPs;
			BYTE ucOptLen = 0;
			USHORT usType;
			BYTE ucNumAddrs = 0;
			USHORT usTypeNO;
			WORD wOptionsIndex = 0;

			memset(ucOptBuf, 0, sizeof(ucOptBuf));

			//Read Server List from registry.
			ProxyReg.GetServerList(&saServers);
			iNumItems = saServers.GetSize();

			for (int i=0; i<iNumItems; i++)
			{
				ParseBootServers(saServers[i], &usType, &ucNumAddrs, &IPs);
				usTypeNO = htons(usType);
				memcpy(&ucOptBuf[wOptionsIndex], &usTypeNO, 2);
				wOptionsIndex += 2;
				memcpy(&ucOptBuf[wOptionsIndex], &ucNumAddrs, 1);
				wOptionsIndex += 1;
				ucOptLen += 3;
				CString strtmp1, strSrc;
				CString strtmp2;
				int iSepPos;
				strtmp2 = IPs;
				for (int j=0; j < ucNumAddrs; j++)
				{
					strSrc = strtmp2;
					iSepPos = strSrc.find(',');
					if (iSepPos != -1)
					{
						strtmp1 = strSrc.substr(0, iSepPos);
						strtmp2 = strSrc.substr(iSepPos+1, strSrc.length());
					}
					else
					{
						strtmp1 = strSrc;
					}
					CIPAddr cipData(strtmp1);
					DWORD dwTmp = cipData.GetIP();
					memcpy(&ucOptBuf[wOptionsIndex], &dwTmp, 4);
					wOptionsIndex += 4;
					ucOptLen += 4;
				}
			}
			// Make sure we have a server list to include in the options.`
			if (ucOptLen > 0)
			{
				// We have a server list so include it in the options.
				ReplyPkt.Add43Option(PXE_BOOT_SERVERS, ucOptBuf, ucOptLen);
			}
		}
	}

	// BOOT SERVER
	// Do we return Network Boot Program (NBP) options?
	// We still need to process a packet here even if it a proxy only req as older
	// cliens expect to receive a file name for bstrap 0
	if ( (f_bBootServerRunning && bBootServerReq) || bProvideBstrapFileName)
	{
		CString strOptionsDll;
		ucLen = ClientPkt.GetOption(DHCP_SYSARCH, ucOptBuf, sizeof(ucOptBuf) );
		if (ucLen == 2)
		{
			usArch =  ntohs(*(USHORT*) &ucOptBuf[0] );
		}
		ucLen = ClientPkt.GetOption(DHCP_NICIF, pucNicInterface, sizeof(pucNicInterface) );
		if (!ucLen)
		{
			// Fill up the UNDI type and major, minor version numbers
			pucNicInterface[0] = 1;
			pucNicInterface[1] = 2;
			pucNicInterface[2] = 1;
		}
		ucLen = ClientPkt.Get43Option(PXE_BOOT_ITEM, ucOptBuf, sizeof(ucOptBuf) );
		if (ucLen == 4)
		{
			usService = ntohs(*(USHORT*) &ucOptBuf[0] );
			usLayer = ntohs(*(USHORT*) &ucOptBuf[2]);
			parserDbg.write("PxeParser: PXE_BOOT_ITEM type=%d layer=%d\n", usService, usLayer);
		} else
		{
			usService = 0;
			usLayer = 0;
		}

		// Return PXE_BOOT_ITEM
		*(USHORT*)&ucOptBuf[0] = htons(usService);
		*(USHORT*)&ucOptBuf[2] = htons(usLayer);

		ReplyPkt.Add43Option(PXE_BOOT_ITEM, ucOptBuf, 4);

		// Point the client back to this server for TFTP
		ReplyPkt.ServerIP(ulServerIP);

		BYTE CredentialTypesList[32];
		BOOL bCredentialTypeReqd = TRUE; 
		int nCredLen = 0;
		if (usLayer & 0x8000)
		{
			nCredLen = ClientPkt.Get43Option(PXE_CREDENTIAL_TYPES, CredentialTypesList, sizeof(CredentialTypesList) );
		}
		else
		{
				bCredentialTypeReqd = FALSE;
		}

		bResult = FALSE;
		unsigned int nSelectedCredType;
		if (bCredentialTypeReqd)
		{
			bResult = ProxyReg.GetImageFile(usArch, pucNicInterface, usService, usLayer, 
				CredentialTypesList, nCredLen, &strBootFile, &nSelectedCredType);
		}
		else
		{
			bResult = ProxyReg.GetImageFile(usArch, pucNicInterface, usService, usLayer, 
				NULL, 0, &strBootFile, NULL);
		}

		if (bResult)
		{	// Insert filename into bootfile field.
			// Convert to a BYTE array.
			bSendResponse = TRUE;
			// convert the back-slashes to forward slashes
			strcpy((char *)ucOptBuf, strBootFile);
			int nslen = strBootFile.length();
			for (int jj=0; jj < nslen; jj++)
			{
				if (ucOptBuf[jj] == '\\')
					ucOptBuf[jj] = '/';
			}
			ReplyPkt.SetBootFileName(ucOptBuf, nslen );
			if (bCredentialTypeReqd)
			{
				// Add the selected credential type to the response.
				unsigned int num = htonl(nSelectedCredType);
				ReplyPkt.Add43Option(PXE_CREDENTIAL_TYPES, (BYTE*)&num, 4);
			}

			//PXE_MTFTP_IP

			if (IsMulticastEnabled())
			{
				// Insert MCast IP address for the file
				DWORD dwMtftpIp;
				// if GetMulticastAddress fails it will not modify dwMtftpIp.
				// GetMulticastAddress will fail if there is not a multicast address
				// assigned to this file.  In that case, none of the multicast options
				// will be included in the packet.
				if  (GetMulticastAddress(&dwMtftpIp, (char*)(strBootFile.data())))
				{
					// Add file transfer options.
					//PXE MTFTP IP
					ReplyPkt.Add43Option(PXE_MTFTP_IP, (BYTE*)&dwMtftpIp, 4);

					//PXE MTFTP CPORT
					USHORT usMtftpCport = GetClientPort();
					ReplyPkt.Add43Option(PXE_MTFTP_CPORT, (BYTE *)&usMtftpCport, 2);

					//PXE MTFTP SPORT
					USHORT usMtftpSport = GetServerPort() ;
					ReplyPkt.Add43Option(PXE_MTFTP_SPORT, (BYTE *)&usMtftpSport, 2);

					//PXE_MTFTP_TMOUT
					BYTE ucMtftpTmout = GetOpenTimeout();
					ReplyPkt.Add43Option(PXE_MTFTP_TMOUT, (BYTE *)&ucMtftpTmout, 1);

					//PXE_MTFTP_DELAY
					BYTE ucMtftpDelay = GetReopenDelay();
					ReplyPkt.Add43Option(PXE_MTFTP_DELAY, (BYTE *)&ucMtftpDelay, 1);
				} //	if  (MTFTPDReg.GetMulticastAddress(dwMtftpIp, strBootFile))
			}// if (IsMulticastEnabled())

		} else // if (bResult)
		{
			// We don't support the requested image, don't send a packet.
			if (!bSendResponse)
			{
				if(parserDbg.DebugOn())
				{
					parserDbg.write("PxeParser: Sending no response as we don't want this packet\n");

				}
				return PXE_PACKET_SENT;
			}
		}

		// NBP OPTIONS
		// Convert the DHCP class object to a BYTE array.
		usMaxPacketSize = ReplyPkt.PacketLength() + 512;
		pucDhcpPacket = new (BYTE [usMaxPacketSize]);
		memset(pucDhcpPacket, 0, sizeof(pucDhcpPacket));
		usPacketSize = usMaxPacketSize;
		ReplyPkt.GetPacket(pucDhcpPacket, usPacketSize);

		// Insert NBP options as appropriate.
		// First insert dynamically defined options
		bResult = ProxyReg.GetServiceOptionsDll(usArch, pucNicInterface, usService, &strOptionsDll);
			parserDbg.write("OptionsDLL = %s\n", strOptionsDll.data());
		if (bResult)
		{	// Invoke Options Dll
			void *handle;
			handle = dlopen(strOptionsDll.data(), RTLD_LAZY);
			parserDbg.write("OptionsDLL = %s\n", strOptionsDll.data());
			
			// Make sure we provide options only for bstrap
			if (handle)
			{
				OptFunc = (PXE_OPTIONS_FUNC)dlsym(handle, OPTIONS_FUNCTION);
				const char* dlerr= dlerror();
				if (dlerr == NULL) 
				{
					//Prepare option structure to send to DLL function.
					// Point to client buffer and size.
					OptStruct.ucClientPkt = pd.ucPacket;
					OptStruct.usClientSize = (USHORT) pd.nPacketLength;

					OptStruct.ucReplyPkt = pucDhcpPacket;
					OptStruct.usReplySize = usPacketSize;
					OptStruct.usReplyMaxSize = usMaxPacketSize;

					OptStruct.usReceivePort = pd.usReceivePort;
					OptStruct.siClientAddr = pd.siClientAddr;

					OptFunc( &OptStruct );

					usPacketSize = OptStruct.usReplySize;
					usMaxPacketSize = OptStruct.usReplyMaxSize;
				}
				dlclose(handle);
			} else
			{
				if (parserDbg.DebugOn())
				{
					parserDbg.write("PxeParser: Unable to map options function.\n");
				}
			}

		} else
		{
			if (parserDbg.DebugOn())
			{
				parserDbg.write("PxeParser:: Unable to load options DLL.\n");
			}
		}
		// Next, look for statically defined options.
		BYTE* pucOptions = NULL;
		DWORD dwOptionsSize = 0;

		// Getting service options is not supported at this time.
#if 0
		bResult = ProxyReg.GetServiceOptions(usArch, pucNicInterface,
																usService, &pucOptions, &dwOptionsSize);
		if (bResult)
		{
			// Insert service options
			if ( usPacketSize + dwOptionsSize <= usMaxPacketSize )
			{
				// Copy over the closing 0xFF
				memcpy(&pucDhcpPacket[usPacketSize-1], pucOptions, dwOptionsSize);
				// Replace the closing 0xFF
				pucDhcpPacket[usPacketSize + dwOptionsSize -1] = 0xFF;
				usPacketSize += dwOptionsSize;
			}
		}
#endif
	}
	// Send packet to the client.
	/////////// Send the response. ////////////////////

	if (!bSendResponse)
	{
		if(parserDbg.DebugOn())
		{
			parserDbg.write("PxeParser: Not sending any response\n");
		}
		return PXE_PACKET_SENT;
	}
	SOCKET sendSocket;  
	SOCKADDR_IN toSockAddr;
	SOCKADDR_IN siSendAddr;
	int status;	
	int numSent=0;
	int nOptVal = 1;



	sendSocket=socket(AF_INET, SOCK_DGRAM, 0);

	if(sendSocket == INVALID_SOCKET)
	{
		parserDbg.logError(ERRLOG_SOCKET_ERROR, "PxeParser: Unable to create socket Error=0x%x\n", WSAGetLastError());
		return PXE_PACKET_SENT;           
	}

	nOptVal = -1;
	int nret = setsockopt(sendSocket, SOL_SOCKET, SO_REUSEADDR,
							(char *) &nOptVal, sizeof(nOptVal) );
	if (nret !=0 )
	{
		parserDbg.logError(ERRLOG_SOCKET_ERROR, "PxeParser: Socket Error while attempting to set " \
				"reuse addr option: Error=0x%x\n", WSAGetLastError());

		closesocket(sendSocket);
		return PXE_PACKET_SENT;
	}

	siSendAddr.sin_port = htons(0); //TODO previously was 67
	siSendAddr.sin_family = AF_INET;
	siSendAddr.sin_addr.s_addr = INADDR_ANY;

	status = bind(sendSocket, (LPSOCKADDR) &siSendAddr, sizeof(siSendAddr));
	if (status == SOCKET_ERROR)
	{
		BYTE* pucIP = (BYTE*)&siSendAddr.sin_addr.s_addr;
		parserDbg.logError(ERRLOG_SOCKET_ERROR, "PxeParser: Socket Error while binding DHCP Socket: 0x%x, Port=%d, IP=%u-%u-%u-%u\n",
							WSAGetLastError(), 67, pucIP[0], pucIP[1],
							pucIP[2], pucIP[3]);
		closesocket(sendSocket);
		return PXE_PACKET_SENT;
	}

	toSockAddr = pd.siClientAddr;
	if (parserDbg.DebugOn())
	{
		BYTE* pucIP = (BYTE*)&toSockAddr.sin_addr.s_addr;
		parserDbg.write("PxeParser: Socket and IP address from client: %d",
							ntohs(toSockAddr.sin_port)	);

		parserDbg.write("PxeParser: Client's IP=%u-%u-%u-%u\n",
							 pucIP[0], pucIP[1], pucIP[2], pucIP[3]);
	}

	// If client does not have an IP then Broadcast response.
	if (pd.usReceivePort == 67)  // Received broadcast
	{
		if (ClientPkt.ClientIP() != 0)
		{
			// We have a valid client with non-zero 'ciaddr'. So, we must be able to unicast a packet
			// to this client.
			if (parserDbg.DebugOn())
			{
				parserDbg.write("PxeParser: Unicasting the response packet to the client\n");
			}

			toSockAddr.sin_port = htons(BOOTP_CPORT);
			toSockAddr.sin_addr.s_addr = ClientPkt.ClientIP();
		}
		else if (ReplyPkt.GatewayIP() != 0)
		{
			// A router may have sent it to us
			// so send it back to the router to forward it. 
			toSockAddr.sin_port = htons(BOOTP_SPORT);
			toSockAddr.sin_addr.s_addr = ReplyPkt.GatewayIP(); 

			if (parserDbg.DebugOn())
			{
				parserDbg.write("PxeParser: Client has a gateway IP.  Sending to port 67.\n");
			}
		} 
		else
		{
			// No gateway and no client IP so broadcast it to the BOOTP client
			// port.
			toSockAddr.sin_port = htons(BOOTP_CPORT);
			//toSockAddr.sin_addr.s_addr = (inet_addr("128.128.255.255"));
			toSockAddr.sin_addr.s_addr = ProxyReg.GetBroadcastIPAddress(); 

			// Set the broadcast flag so the socket will send broadcasts. 
			int on=1;
			int err = 0;
			err = setsockopt(sendSocket, SOL_SOCKET, SO_BROADCAST, (char *)&on, sizeof(on));
			if (err)
			{
				parserDbg.write("PxeParser: Socket error 0x%x while setting broadcast option: \n", 
					WSAGetLastError());
				// Clean-up and go home.
				closesocket(sendSocket);
				delete[] pucDhcpPacket;
			}

			if (parserDbg.DebugOn())
			{
				parserDbg.write("PxeParser: Broadcasting the response packet to the client\n");
			}
		}

	}  else  // Received mulitcast or unicast on port 4011.
	{	// Unicast to the client's address.
			if (parserDbg.DebugOn())
			{
				parserDbg.write("PxeParser: Received packet on port 4011\n");
			}

			toSockAddr = pd.siClientAddr;
//			toSockAddr.sin_port = htons(4011);

	}
	// The client has an IP so unicast to the client's port.
	if (parserDbg.DebugOn())
	{
		parserDbg.write("PxeParser: Sending packet to client on port %d",
							ntohs(toSockAddr.sin_port)	);
		BYTE* pucIP = (BYTE*)&toSockAddr.sin_addr.s_addr;
		parserDbg.write(" IP=%u-%u-%u-%u\n",
							 pucIP[0], pucIP[1], pucIP[2], pucIP[3]);
	}

	{
		numSent = sendto(sendSocket,
						(const char *)pucDhcpPacket, usPacketSize, 0,
						(struct sockaddr *)&toSockAddr, sizeof(struct sockaddr));

		if (numSent <= 0)
		{
			parserDbg.logError(ERRLOG_SOCKET_ERROR, "PxeParser: sendto() to client %s failed with error = 0x%x\n",
					inet_ntoa(toSockAddr.sin_addr),
					WSAGetLastError());
		} else
		{
			if(parserDbg.DebugOn())
			{
				parserDbg.write("PxeParser: Sendto sent %d bytes to %s.\n",
									numSent, inet_ntoa(toSockAddr.sin_addr));
			}
		}
	}

	// Clean-up and go home.
	closesocket(sendSocket);
	delete[] pucDhcpPacket;
	return PXE_PACKET_SENT;

}

void ParseBootServers(CString p_strServer, USHORT *p_usType, BYTE *p_ucLen,
					  CString *pIPs)
{
	CString strSrc = p_strServer;
	CString strtmp1;
	CString strtmp2;
	int iSepPos;
	BYTE ucNumIps = 0;

	iSepPos = strSrc.find(',');
	strtmp1 = strSrc.substr(0,iSepPos);
	strtmp2 = strSrc.substr(iSepPos+1, strSrc.length());
	*p_usType = (USHORT) atoi(strtmp1);
	
	strSrc = strtmp2;
	iSepPos = strSrc.find(',');
	strtmp1 = strSrc.substr(0, iSepPos);
	strtmp2 = strSrc.substr(iSepPos+1, strSrc.length());
	ucNumIps = (BYTE) atoi(strtmp1);
	*p_ucLen = ucNumIps;
	*pIPs = strtmp2;

	/*
	for (int i=0; i < ucNumIps - 1; i++)
	{
		strSrc = strtmp2;
		iSepPos = strSrc.find(',');
		strtmp1 = strSrc.Left(iSepPos);
		strtmp2 = strSrc.Right(strSrc.GetLength() - iSepPos -1);
		p_SrvrIPs->SetAtGrow(i, inet_addr(strtmp1));
	}
	*/

	// Get the last IP address

}

// This routine is not used in Linux PXE services
BOOL GetHostIPAddress(BYTE *bHostIP, int size)
{
	TCHAR szHostName[128];
	BYTE bLoopBack[10];
	unsigned long ulLoopBack;

	if(size < IPLEN)
		return FALSE;
	bHostIP[0] = 128;
	bHostIP[1] = 128;
	bHostIP[2] = 0;
	bHostIP[3] = 20;

	return TRUE;
#ifdef GETHOSTADDR	
	ulLoopBack = inet_addr("127.0.0.1");
	memcpy(bLoopBack, &ulLoopBack, IPLEN);

	if (gethostname(szHostName, sizeof(szHostName)) == SOCKET_ERROR)
	{
		return FALSE;
	}
	
	hostent *phosts;
	// Loop until a valid IP address is obtained.
	while(1)
	{
		phosts = gethostbyname(szHostName);
		if(phosts == NULL)
		{
			return FALSE;
		}

		memcpy(bHostIP, phosts->h_addr_list[0], IPLEN);

		if ( memcmp(bHostIP, bLoopBack, IPLEN) == 0)
		// No IP address has been assigned to this server 
		// so sleep for 1/2 a second and try again.
			Sleep(500);
		else
			return TRUE;
	}
#endif
}    
