
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#include <sys/signal.h>

#include "pxemisc.h"
#include "cpxedebug.h"
#include "parsedll.h"
#include "cipaddr.h"
#include <dlfcn.h>

UINT f_uiNumSuccessParsers = 0;
BOOL bControlTFTP = FALSE;

//FILE* fpDebug = NULL;
CProxyRegistry ProxyReg;
BYTE ucPacket[BOOTP_EXVENDOR];
// HELPER FUNCITONS
extern BOOL InitRegistry();
BOOL LoadParsingDlls();
void SendToClients(CPacketData ClientData);
int main_function();


struct _DllStruct
{
	void* hLib;
	PXE_PARSER_FUNC fnc;
};

typedef struct _DllStruct DLLSTRUCT;
DLLSTRUCT* pDlls;


CPxeDebug*	serviceDbg;

void killSelf(int signo) {
    unlink("/var/run/pxe.pid");
    exit(0);
}

int main()
{
	struct ip_mreq stIpMreq;

	SOCKET scDhcp = -1;
	SOCKET scBinl = -1;
	SOCKADDR_IN siDhcp;
	SOCKADDR_IN siBinl;
	SOCKADDR_IN siFrom;

	ULONG wDiscoveryAddr = 0;
	ULONG wServerAddr = 0;
	DWORD saSize = 0;
	CPacketData ClientData;
	BOOL bMasterProxy = FALSE;
	int status;
	fd_set rfd;

	printf("Intel PXE daemon, PXE-2.0 Beta-1 (build 001)\n"
		"Copyright (C) 1999  Intel Corporation\n");

	if (fork() != 0)
		return 0;

	signal(SIGTERM, killSelf);

	{
	    FILE * f;

	    f = fopen("/var/run/pxe.pid", "w");
	    fprintf(f, "%d\n", getpid());
	    fclose(f);
	}

	if (ProxyReg.GetDebugFlag())
	{
		//fpDebug = fopen("c:\\DBBootDHCP.txt", "w");
	}

	serviceDbg = new CPxeDebug();

	serviceDbg->init(PXE_SERVICES_DEBUG);

	bMasterProxy = ProxyReg.MasterProxyFlag();
	// If we are not a "Master Proxy" then get our configuration from some other server.
	if (! bMasterProxy)
	{
		serviceDbg->logError(ERRLOG_GENERIC_ERROR, "Master Proxy flag must be 1\n");
	}

	// Start the TFTP Service

	wServerAddr = ProxyReg.GetDiscoverySrvrIP();
	if (wServerAddr == 0)
		wServerAddr = INADDR_ANY;
	else
	{

		if (serviceDbg->DebugOn())
		{
			BYTE* pucIP = (BYTE*)&wServerAddr;
			serviceDbg->write("regsitry: Discovery_Srvr_IP=%u-%u-%u-%u\n",
								 pucIP[0], pucIP[1],	pucIP[2], pucIP[3]);
		}
	}
	// Attempt to load the parsing dlls.
	if( ! LoadParsingDlls() )
	{
		return 0;
	}

	// Initialize Sockets
	// Multicast Discovery Address - may not be set.
	 wDiscoveryAddr = ProxyReg.GetDiscoveryAddr();

	 // Clear the socket addresses.
	memset(&siDhcp, 0, sizeof(siDhcp));
	memset(&siBinl, 0, sizeof(siBinl));
	// Initialize the DHCP socket
	if (ProxyReg.UseDHCPPort())
	{
		siDhcp.sin_port = htons(67);
		siDhcp.sin_family = AF_INET;
		siDhcp.sin_addr.s_addr = wServerAddr; //INADDR_ANY;

		scDhcp = socket(AF_INET, SOCK_DGRAM, 0);
		if (scDhcp == INVALID_SOCKET)
		{
			serviceDbg->logError(ERRLOG_SOCKET_ERROR, "Error creating DGRAM socket: %d\n", 
				WSAGetLastError());
			return 0;
		}

		status = bind(scDhcp, (LPSOCKADDR) &siDhcp, sizeof(siDhcp));
		if (status == SOCKET_ERROR)
		{
			BYTE* pucIP = (BYTE*)&siDhcp.sin_addr.s_addr;
			serviceDbg->write("Error 0x%x while binding DHCP Socket on Port=%d, IP=%u-%u-%u-%u\n",
								WSAGetLastError(), 67, pucIP[0], pucIP[1],
								pucIP[2], pucIP[3]);
			closesocket(scDhcp);
			// This is not a fatal condition for the proxy as it can still listen 
			// on port 4011. So, don't quit and continue.
			
			// Initialize the scDhcp back to zero
			scDhcp = 0;
		}
		if (serviceDbg->DebugOn() && scDhcp)
		{
			BYTE* pucIP = (BYTE*)&siDhcp.sin_addr.s_addr;
			serviceDbg->write("TCP INFO DHCP Socket bound to: Port=%d, IP=%u-%u-%u-%u\n",
								67, pucIP[0], pucIP[1],	pucIP[2], pucIP[3]);
		}
	}
	// Initialize the BINL socket
	siBinl.sin_port = htons(4011);
	siBinl.sin_family = AF_INET;
	siBinl.sin_addr.s_addr = wServerAddr; //INADDR_ANY;
	scBinl = socket(AF_INET, SOCK_DGRAM, 0);
	if (scBinl == INVALID_SOCKET)
	{
		serviceDbg->logError(ERRLOG_SOCKET_ERROR,
			"PxeService: Error 0x%x creating Datagram socket\n", WSAGetLastError());
		return 0;
	}

	status = bind(scBinl, (LPSOCKADDR) &siBinl, sizeof(siBinl));
	if (status == SOCKET_ERROR)
	{
		BYTE* pucIP =  (BYTE*)&siDhcp.sin_addr;
		serviceDbg->logError(ERRLOG_SOCKET_ERROR,
			"PxeService: Error 0x%x while binding Socket on Port=%d, IP=%u-%u-%u-%u\n", 
			WSAGetLastError(), 4011, pucIP[0], pucIP[1], pucIP[2], pucIP[3]);

		closesocket(scBinl);
		return 0;
	}
	if (serviceDbg->DebugOn())
	{
		BYTE* pucIP = (BYTE*)&siBinl.sin_addr.s_addr;
		serviceDbg->write("Socket INFO BINL Socket No, %d, bound to: Port=%d, IP=%u-%u-%u-%u\n",
							scBinl, 4011, pucIP[0], pucIP[1],	pucIP[2], pucIP[3]);
	}
	// Join multicast group if a multicast address is specified.
//#ifdef 1
	if (wDiscoveryAddr)
	{
		stIpMreq.imr_multiaddr.s_addr = wDiscoveryAddr;
		CString csIp;
		ProxyReg.GetSystemIPAddress(&csIp);
		CIPAddr cipaddr(csIp);
		DWORD cipdword = cipaddr.GetIP();
		stIpMreq.imr_interface.s_addr = cipdword;

		int nret = setsockopt(scBinl, IPPROTO_IP, IP_ADD_MEMBERSHIP,
								(char *) &stIpMreq, sizeof(stIpMreq) );
		if (nret == SOCKET_ERROR)
		{
			serviceDbg->logError(ERRLOG_SOCKET_ERROR,
				"PxeService: Error 0x%x while attempting to add multicast membership\n", 
				WSAGetLastError());
			closesocket(scBinl);
			return 0;
		}
	}
	if (serviceDbg->DebugOn())
	{
		BYTE* pucIP = (BYTE*)&(stIpMreq.imr_multiaddr.s_addr );
		serviceDbg->write("Joined Mcast IP: %u-%u-%u-%u\n",
							pucIP[0], pucIP[1],	pucIP[2], pucIP[3]);
	}
//#endif

#ifdef GET_MACHINE_NAME
	// Get the machine Info by creating a thread. We are creating a thread to do this
	// as it takes 15 seconds for the call to fail if we are not in a domain. But for the time
	// being, we are still going to wait for the thread to finish (why need a thread then !!)
	// because several boot servers like DOSUNDI, APITEST etc require these parameters to 
	// function properly. May be one of these days, we can remove this wait.
	
	{
		DWORD dwTid;
		HANDLE hThread = CreateThread (NULL, 0, GetDomainInfoProc, (LPVOID)&ProxyReg, 0, &dwTid);
		if (hThread == NULL)
		{
			serviceDbg->logError(ERRLOG_GENERIC_ERROR, "PxeService: Unable to create the GetDomaininfo Thread.\n");
		}
		else
		{
			serviceDbg->write("Waiting for GetDomainInfo to execute\n");
			WaitForSingleObject (hThread, INFINITE);
			serviceDbg->write("GetDomainInfo done..\n");
			CloseHandle(hThread);
		}
	}
#endif
	while (1)
	{
		int n = 0;
		int numrcv= 0;
		FD_ZERO(&rfd);
		if (scDhcp >= 0)
			FD_SET(scDhcp, &rfd);
		if (scBinl >= 0)
			FD_SET(scBinl, &rfd);
		serviceDbg->write("Waiting for another packet from client\n");
		// The first parameter to select must be the highest 
		// numbered descriptor + 1
		int nMax = (scDhcp > scBinl) ? scDhcp : scBinl;
		nMax++;
		n = select(nMax, &rfd, 0, 0, NULL);
		if (n == SOCKET_ERROR)
		{
			serviceDbg->logError(ERRLOG_SOCKET_ERROR,
				"PxeService: Error 0x%x in Socket select call\n", WSAGetLastError());
			return 0;
		}

		if ((n >= 1) && (scDhcp >= 0) && FD_ISSET(scDhcp, &rfd) )
		{

			n--;
			saSize = sizeof(SOCKADDR);
			numrcv = recvfrom(scDhcp, (char*)ucPacket, 1024,
										0,(SOCKADDR *)&siFrom, &saSize);
			if (numrcv == SOCKET_ERROR)
			{
				DWORD err = WSAGetLastError();
				serviceDbg->logError(ERRLOG_SOCKET_ERROR, "PxeService: WinSock error 0x%x on " \
					"UDP recvfrom() on Port 67\n", err);
				serviceDbg->logError(ERRLOG_SOCKET_ERROR, "PxeService is being stopped " \
					"because of the UDP read error\n");
					break;
			}
			if (serviceDbg->DebugOn())
			{
				serviceDbg->write("Received DHCP Packet with %d Bytes on port 67\n", numrcv);
			}
			ClientData.usReceivePort = (USHORT) 67;
			memset(ClientData.SendBuf, 0, sizeof(ClientData.SendBuf));
			memcpy(ClientData.SendBuf,  ucPacket, numrcv);
			ClientData.numSent = numrcv;
			ClientData.toSockAddr = siFrom;
			SendToClients(ClientData);
		}
		if ((n >= 1) && (scBinl >= 0) && FD_ISSET(scBinl, &rfd) )
		{
			n--;
			saSize = sizeof(SOCKADDR);
			numrcv = recvfrom(scBinl, (char*)ucPacket, MAXDISCLEN,
										0,(SOCKADDR *)&siFrom, &saSize);
			if (numrcv == SOCKET_ERROR)
			{
				DWORD err = WSAGetLastError();
					serviceDbg->logError("PxeService: WinSock error 0x%x on " \
						"UDP recvfrom() on Port 4011\n", err);
					serviceDbg->logError("PxeService: PxeService is being stopped " \
						"because of the UDP read error\n");
					break;
			}
			if (serviceDbg->DebugOn())
			{
				serviceDbg->write("Received DHCP Packet with %d Bytes on port 4011\n", numrcv);
			}
			ClientData.usReceivePort = (USHORT) 4011;
			memset(ClientData.SendBuf, 0, sizeof(ClientData.SendBuf));
			memcpy(ClientData.SendBuf,  ucPacket, numrcv);
			ClientData.numSent = numrcv;
			ClientData.toSockAddr = siFrom;
			SendToClients(ClientData);
		}

	} // Read Loop
	return 1;
}

//extern "C" DWORD
//PxeParser(PXE_PARSER_DATA *lpParam);

void
SendToClients(	CPacketData ClientData)
{
#if 0
	PXE_PARSER_DATA ParserData;
	UINT i = 0;
	DWORD dwResult = 0;
	BYTE* pucTmp = NULL;
	BOOL bPacketHandled = FALSE;
	static int nNumPackets = 0;


	nNumPackets++;
	// Allocate a BYTE array for the packet.
	pucTmp = new (BYTE[ClientData.numSent]);

		// Make a fresh copy of everything in case the last parser modified it.
		ParserData.usReceivePort = ClientData.usReceivePort;
		ParserData.ucPacket = pucTmp;
		memcpy(ParserData.ucPacket, ClientData.SendBuf, ClientData.numSent);
		ParserData.nPacketLength = ClientData.numSent;
		ParserData.siClientAddr = ClientData.toSockAddr;
		PxeParser(&ParserData);

	delete[] pucTmp;
#endif

	PXE_PARSER_DATA ParserData;
	UINT i = 0;
	DWORD dwResult = 0;
	BYTE* pucTmp = NULL;
	BOOL bPacketHandled = FALSE;
	static int nNumPackets = 0;

	nNumPackets++;
	// Allocate a BYTE array for the packet.
	pucTmp = new (BYTE[ClientData.numSent]);

	for (i = 0; i < f_uiNumSuccessParsers; i++)
	{
		// Make a fresh copy of everything in case the last parser modified it.
		ParserData.usReceivePort = ClientData.usReceivePort;
		ParserData.ucPacket = pucTmp;
		memcpy(ParserData.ucPacket, ClientData.SendBuf, ClientData.numSent);
		ParserData.nPacketLength = ClientData.numSent;
		ParserData.siClientAddr = ClientData.toSockAddr;
		dwResult = pDlls[i].fnc(&ParserData);
		if (dwResult == PXE_PACKET_SENT || dwResult == PXE_PARSER_ERROR)
		{
			bPacketHandled = TRUE;
			break;
		}
	}

	if (serviceDbg->DebugOn())
	{
		if (bPacketHandled)
		{
			if (dwResult == PXE_PACKET_SENT)
			{
					serviceDbg->write("PxeService: Packet #%d handled by Parser %d (PXE_PACKET_SENT)\n", nNumPackets, i);
			}
			if (dwResult == PXE_PARSER_ERROR)
			{
					serviceDbg->write("PxeService: Packet #%d handled by Parser %d (PXE_PARSER_ERROR)\n", nNumPackets, i);
			}
		} else
		{
			serviceDbg->write("PxeService: Packet #%d was NOT handled by any parsers\n", nNumPackets);
		}
	}

	delete[] pucTmp;
			

}

BOOL
LoadParsingDlls()
{
	CStringArray strarParsers;
	CString csDllPath;
	CString csFuncName;
	UINT uiNumParsers = 0;
	BOOL bResult = FALSE;
	UINT i = 0;
	uiNumParsers = ProxyReg.GetClients(&strarParsers);
	if (serviceDbg->DebugOn())
	{
		serviceDbg->write("LoadParsingDlls:  %d Parsing Dlls specified in Registry.\n", uiNumParsers);
		for (UINT j = 0; j< uiNumParsers; j++)
			serviceDbg->write("                 %d: %s.\n", j, strarParsers[j].data());
	}
	if (uiNumParsers)
	{
		pDlls = new (DLLSTRUCT[uiNumParsers]);
		for (i = 0; i < uiNumParsers; i++)
		{
			bResult = ProxyReg.GetParserFunction( strarParsers[i], &csDllPath, &csFuncName);
			if (bResult == FALSE)
			{
				// ERROR MESSAGE - client DLL path or function missing.
				serviceDbg->logError(ERRLOG_GENERIC_ERROR, 
					"LoadParsingDlls:  Failed to find Dll Path or Funcion Name for %s.\n", strarParsers[i].data());
			} else
			{
				// Load library

				pDlls[f_uiNumSuccessParsers].hLib = dlopen(csDllPath.data(), RTLD_LAZY);
				if (!pDlls[f_uiNumSuccessParsers].hLib)
				{
					// ERROR MESSAGE - unable to load client DLL.
					serviceDbg->logError(ERRLOG_GENERIC_ERROR, "LoadParsingDlls:  Failed to load Dll Library for %s.\n", strarParsers[i].data());
				} else
				{
					// Map Function
					pDlls[f_uiNumSuccessParsers].fnc = (PXE_PARSER_FUNC) dlsym(pDlls[f_uiNumSuccessParsers].hLib, csFuncName);
					const char* dlerr= dlerror();

					if (dlerr != NULL)
					{
						// ERROR MESSAGE - unable to map function.
						serviceDbg->logError(ERRLOG_GENERIC_ERROR, "LoadParsingDlls:  Failed to map Funcion Name for %s.\n", strarParsers[i].data());
					} else
					{
						// Call the 'Initialize' function if it exists
						typedef DWORD (*PXE_INIT_FUNC)();
						PXE_INIT_FUNC pInitFunc;
						pInitFunc = (PXE_INIT_FUNC)dlsym(pDlls[f_uiNumSuccessParsers].hLib, "Initialize");
						if (pInitFunc)
						{
							// call the initialize function so that the DLL can initialize itself
							(*pInitFunc)();
						}
						f_uiNumSuccessParsers++;
					}
				}
			}
		}
	}
	if (f_uiNumSuccessParsers == 0)
	{
printf("failed\n");
		// ERROR MESSAGE - No clients to process the packets.
		serviceDbg->logError(ERRLOG_GENERIC_ERROR, "LoadParsingDlls:  Failed to find any parsing Dlls. \n");
		return FALSE;
	}
	return TRUE;
}

